package com.blackledge.david.game;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

public interface MultiplayerGame {
	
	public static final int GAME_NEW_GAME = -1;
	public static final int GAME_VS_ANY_PLAYER = -2;
	public static final int GAME_VS_COMPUTER = -3;

	public static class GameInfo {
		public int gameID;
		public List<PlayerState> players;
		@Override
		public boolean equals(Object obj) {
			if(obj instanceof GameInfo) {
				GameInfo g = (GameInfo) obj;
				return(gameID == g.gameID) && 
						((players == null && g.players == null) || 
								(players != null && players.equals(g.players)));
			}
			return super.equals(obj);
		}
	}

//	/**
//	 * 
//	 * @param gameID id of game to join, or -1 to create a new game.
//	 * @param client
//	 * @param playerIdentification id and public name of player that is joining
//	 * @return gameID of game joined
//	 * @throws GameException if game could not be joined/created.
//	 */
//	public int connectPlayerClient(int gameID, GameStateListener client, PlayerState playerIdentification) 
//			throws GameException;
//	public void connectSpectatorClient(int gameID, GameStateListener client, PlayerState spectatorIdentification)
//			throws GameException;
//	
//	public void disconnectPlayerClient(int gameID, GameStateListener client, PlayerState playerIdentification)
//			throws GameException;
	
	public boolean playerAction(int gameID, PlayerState playerIdentification, int action, Object details);
	
//	public void connectChatListener(int gameID, ChatListener client);
//	public void disconnectChatListener(int gameID, ChatListener client);

	public void playerChat(int gameID, PlayerState playerIdentification, String message);
	
	public GameInfo[] getGameList();
	
	public int getBoardHeight();
	public int getBoardWidth();
	public int newGame();
	
	public Date getGameUpdated(int gameID);
	public Date getPlayersUpdated(int gameID);
	public Date getChatUpdated(int gameID);
	public Date getBoardUpdated(int gameID);
	
	public BoardState getGameBoard(int gameID);
	public ArrayList<PlayerState> getGamePlayers(int gameID);
	/**
	 * Get the FIRST player that has turn.
	 * @param game
	 * @return
	 */
	public PlayerState getPlayerWithTurn(int gameID);
	public List<PlayerState> getPlayersWithTurn(int gameID);
	
	public ArrayList<Chat> getGameChat(int gameID);
	
	public GameData getGameData(int gameID);

	public static class GameData {
		public GameData(int gameID) {
			this.gameID = gameID;
		}
		public final int gameID;

		public ArrayList<PlayerState> players = new ArrayList<PlayerState>();
		public BoardState boardState;
		public Date lastChanged;
		public Date lastChangedPlayers;
		public Date lastChangedChat;
		public Date lastChangedBoard;
		public ArrayList<Chat> chat = new ArrayList<Chat>();
		public int minimumPlayers = 2;
		public int maximumPlayers = 2;

		public Date lastPlayerAction;

//		ArrayList<GameStateListener> gameListeners = new ArrayList<GameStateListener>();
//		ArrayList<ChatListener> chatListeners = new ArrayList<ChatListener>();
	}

	
	/**
	 * Flag that all elements of the game should be considered changed
	 * @param gameID
	 */
	public void gameUpdated(int gameID);
	public void boardUpdated(int gameID);
	public void chatUpdated(int gameID);
	public void playersUpdated(int gameID);
	
	public void leaveGame(int gameID, PlayerState player) throws GameException;
	public int joinGame(int gameId, PlayerState player) throws GameException;

	public boolean isComputerPlayerAvailable();
}