package com.blackledge.david.html;

import java.net.CookieManager;
import java.net.CookieStore;
import java.net.HttpCookie;
import java.net.URI;
import java.util.List;

/**
 * Inspired by
 * http://stackoverflow.com/questions/16305486/cookiemanager-for-multiple-threads 
 * but all we really needed was a ThreadLocal CookieStore proxy class - that's what this is.
 * 
 * @author David
 * 
 */
public class ThreadLocalCookieStore implements CookieStore {

    private final static ThreadLocal<CookieStore> ms_cookieJars = new ThreadLocal<CookieStore>() {
        @Override
        protected synchronized CookieStore initialValue() { 
        	System.out.println("*** new cookiestore! ***");
        	return (new CookieManager()).getCookieStore(); /*InMemoryCookieStore*/ 
        	}
    };
	
	
	public void add(URI uri, HttpCookie cookie) {
		ms_cookieJars.get().add(uri, cookie);
	}

	public List<HttpCookie> get(URI uri) {
		return ms_cookieJars.get().get(uri);
	}

	public List<HttpCookie> getCookies() {
		return ms_cookieJars.get().getCookies();
	}

	public List<URI> getURIs() {
		return ms_cookieJars.get().getURIs();
	}

	public boolean remove(URI uri, HttpCookie cookie) {
		return ms_cookieJars.get().remove(uri, cookie);
	}

	public boolean removeAll() {
		return ms_cookieJars.get().removeAll();
	}

}
