package com.blackledge.david.html.hme;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Date;
import java.util.Vector;

/**
 * Handle Ads for WebBrowserApp. Uses System.getProperty("ads", "ads.txt").
 */
public class Advertisement {
	public String title;
	public String image;
	public String url;
	public String options;
	public boolean isApp;
	public int appPriority;
	public String genre;
	public boolean hidden;
	//...
	
	public Advertisement(String source) {
		String[] parts = source.split("\t");
		if(parts.length > 0) {
			url = parts[0];
			if(parts.length > 1) {
				title = parts[1];
				if(parts.length > 2) {
					image = parts[2];
					if(parts.length > 3) {
						setOptions(parts[3]);
						if(parts.length > 4) {
							try {
								Integer appPriority = Integer.parseInt(parts[4]);
								this.appPriority = appPriority;
								isApp = true;
							} catch(Exception e) {
								isApp = false;
							}
							if(parts.length > 5) {
								this.genre = parts[5];
								if(parts.length > 6) {
									this.hidden = Boolean.valueOf(parts[6]);
								}
							}
						}
					}
				}
			}
		}
	}

	private void setOptions(String optionstring) {
		WebBrowserOptions o = new WebBrowserOptions(null);
		o.loadOptions(optionstring);
		options = o.getURLOptions();
	}
	
	private static Vector<Advertisement> ads = new Vector<Advertisement>();
	private static long adRefresh = 0;
	
	/**
	 * 
	 * @return ?String title?, URL url, ?String icon?, Integer priority, String genre, options
	 */
	public static Vector<Object[]> getAppAds() {
		Vector<Object[]> result = new Vector<Object[]>();
		synchronized(ads) {
			loadAds(ads);
			for(Advertisement ad : ads) {
				if(ad.isApp) {
					try {
						Object[] item = getAdItem(ad);

						result.add(item);

					} catch (MalformedURLException e) {
						WebBrowserApp.debugOut("ad URL error: " + e.getMessage());
					}
				}
			}
		}
		return result;
	}

	/**
	 * 
	 * @return ?String title?, URL url, ?String icon?, ?appPriority?, ?String genre?, ?options?, ?Boolean hidden?
	 */
	private static Object[] getAdItem(Advertisement ad)
			throws MalformedURLException {
		Object[] item = new Object[7];
		if (ad.title != null && ad.title.length() > 0) {
			item[0] = ad.title;
		}
		if (ad.url != null && ad.url.length() > 0) {
			item[1] = new URL(ad.url);
		}
		if (ad.image != null && ad.image.length() > 0) {
			item[2] = ad.image;
		}
		if (ad.isApp) {
			item[3] = new Integer(ad.appPriority);
		}
		if(ad.genre != null && ad.genre.length() > 0) {
			item[4] = ad.genre.trim();
		}
		if(ad.options != null && ad.options.length() > 0) {
			item[5] = ad.options;
		}
		if(ad.hidden) {
			item[6] = ad.hidden;
		}
		return item;
	}
	/**
	 * Does not include ads where isApp is true.
	 * @return ?String title?, URL url, ?String icon?
	 */
	public static Object[] getRandomAd() {
		synchronized(ads) {
			loadAds(ads);

			if(ads.size() > 0) {
				int sanity = 100;
				Advertisement ad = null;
				do {
					double rand = Math.random(); // number from 0.0 to LESS THAN 1.0
					// rand is less than 1, and floor rounds down if not an integer, so this
					// is guaranteed to be less than size
					int index = (int) Math.floor(rand*ads.size()); 

					if(index < 0 || index >= ads.size()) 
						index = 0;

					ad = ads.get(index);
					if (--sanity < 0) {
						break; // just use the app one.
					}
				} while (ad.isApp);
				try {
					Object[] result = getAdItem(ad);

					return result;
				} catch (MalformedURLException e) {
					WebBrowserApp.debugOut("ad URL error: " + e.getMessage());
				}
			}
		}
		
		try {
			return new Object[] {
					"<"+WebBrowserApp.BROWSER_NAME+" suggested links page>",
					new URL("http://hmedev.wikidot.com/enter-webz-links#main-content"),
					null};
		} catch (MalformedURLException e) {
			return new Object[] {
					"Have a Nice Day!",null,null
			};
		}
	}
	private static void loadAds(Vector<Advertisement> ads) {
		long now = new Date().getTime();
		if (ads.size() > 0 && (now - adRefresh) < 60*60*1000) {
			return;
		}
		adRefresh  = now;
		ads.clear();
		
		String filename = System.getProperty("ads", "ads.txt");
		BufferedReader reader;
		try {
			reader = new BufferedReader(new FileReader(filename));
			for (String line = reader.readLine(); line != null; line = reader.readLine()) {
				if(line.startsWith("#") || line.trim().length() == 0) continue;
				ads.add(new Advertisement(line));
				WebBrowserApp.debugOut("Load Ad: "+line);
			}
			reader.close();
			return;
		} catch (FileNotFoundException e) {
			WebBrowserApp.debugOut("load Ads "+filename+": " + e.getMessage());
		} catch (IOException e) {
			WebBrowserApp.debugOut("load Ads "+filename+": " + e.getMessage());
		}

		// make sure the list isn't empty
		if(ads.size() == 0) {
			ads.add(new Advertisement(
					"http://hmedev.wikidot.com/enter-webz-links#main-content"
							+ "\t" +
							"[" + WebBrowserApp.BROWSER_NAME + " suggested links]"
							+ "\t" +
							""
							+ "\t" +
							"false"
					));
		}
	}
	
}