package com.blackledge.david.html.hme;

import java.awt.Color;

import com.tivo.hme.bananas.BHighlights;
import com.tivo.hme.bananas.BRect;
import com.tivo.hme.bananas.BView;
import com.tivo.hme.sdk.View;

public class BLinkView extends BView {

	private String link;
	private Color[] linkColors;

	public BLinkView(String link, Color[] linkColors, BView parent, int x, int y, int width, int height,
			boolean visible) {
		super(parent, x, y, width, height, visible);
		init(link, linkColors);
	}

	private void init(String link, Color[] linkColors) {
		this.link = link;
		this.linkColors = linkColors;
		setFocusable(true);
		setHighlights();
	}

	private void setHighlights() {
		getHighlights().setWhisperingArrow(BHighlights.H_RIGHT,
				BHighlights.A_RIGHT+10,
				BHighlights.A_CENTER, "right");
		getHighlights().setWhisperingArrow(BHighlights.H_LEFT,
				BHighlights.A_LEFT-10,  
				BHighlights.A_CENTER, "left");
		getHighlights().setWhisperingArrow(BHighlights.H_UP,
				 BHighlights.A_CENTER, 
				 BHighlights.A_TOP-10, "up");
		getHighlights().setWhisperingArrow(BHighlights.H_DOWN,
				 BHighlights.A_CENTER, 
				 BHighlights.A_BOTTOM+10, "down");
		getHighlights().setVisible(BHighlights.H_RIGHT, BHighlights.H_VIS_FALSE);
		getHighlights().setVisible(BHighlights.H_LEFT, BHighlights.H_VIS_FALSE);
		getHighlights().setVisible(BHighlights.H_UP, BHighlights.H_VIS_FALSE);
		getHighlights().setVisible(BHighlights.H_DOWN, BHighlights.H_VIS_FALSE);
	}

	public BLinkView(String link, Color[] linkColors, BView parent, int x, int y, int width, int height) {
		super(parent, x, y, width, height);
		init(link, linkColors);
	}

	@Override
	public boolean handleFocus(boolean isGained, BView gained, BView lost) {
		// make sibling blinkviews unfocusable until we lose focus.
		BLinkView focus = this;
		String link = ((BLinkView)focus).getLink();
		if (link != null && focus.getParent() != null && focus.getParent().getParent() != null) {
			// hierarchy is common parent :: object :: blinkview
			for (int i = 0; i < focus.getParent().getParent().getChildCount(); ++i) {
				View child1 = focus.getParent().getParent().getChild(i);
				for(int j = 0 ; j < child1.getChildCount() ; ++j) {
					View child = child1.getChild(j);
					if (child instanceof BLinkView
							&& link.equals(((BLinkView) child).getLink())) {
						((BLinkView)child).setFocusable(!isGained);
					}
				}
			}
		}
		
		return super.handleFocus(isGained, gained, lost);
	}
	
	public String getLink() {
		return link;
	}
	
	public Color getLinkColor() {
		if(linkColors != null && linkColors.length > 0) {
			return linkColors[0];
		}
		return Color.blue;
	}
	
	public Color getLinkActiveColor() {
		if(linkColors != null && linkColors.length > 1) {
			return linkColors[1];
		}
		return getLinkColor();
	}
	
	public Color getLinkVisitedColor() {
		if(linkColors != null && linkColors.length > 2) {
			return linkColors[2];
		}
		return getLinkColor();
	}
	
	/**
	 * resets the highlights, too
	 */
	@Override
	public void setSize(int width, int height) {
		super.setSize(width, height);
		setHighlights();
	}

}
