package com.blackledge.david.html.hme;

import java.awt.Dimension;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Map;
import java.util.regex.Matcher;

import com.blackledge.david.html.HTMLImageChunk;
import com.tivo.hme.bananas.BView;
import com.tivo.hme.sdk.HmeEvent;
import com.tivo.hme.sdk.IHmeEventHandler;
import com.tivo.hme.sdk.Resource;
import com.tivo.hme.sdk.StreamResource;
import com.tivo.hme.sdk.HmeEvent.ResourceInfo;

/**
 * Handle an image so we can deal with redirect errors, record its dimensions
 * (or permanent error) for future requests, and continue rendering the page
 * that was blocked now that we know its size
 */
class BlockingInlineImageResourceHandler implements
			IHmeEventHandler {
		protected final HTMLBreakdown _htmlBreakdown;
		protected final HTMLImageChunk _chunk;
		protected final BView _view;
		protected final URL _urlBase;
		protected Object IMAGE_ERROR_RESOURCE;
		private boolean redirectedOnce = false;
		private String triedHTTP;

		BlockingInlineImageResourceHandler(HTMLBreakdown htmlBreakdown, HTMLImageChunk chunk,
				BView view, URL urlBase) {
			_htmlBreakdown = htmlBreakdown;
			_chunk = chunk;
			_view = view;
			_urlBase = urlBase;
			IMAGE_ERROR_RESOURCE = _view.getBApp().getContext().getBaseURI().toString()+"image-error.png";
		}

		public void postEvent(HmeEvent event) {
			if(event.getOpCode() == HmeEvent.EVT_APP_INFO) {
		      HmeEvent.ApplicationInfo info = (HmeEvent.ApplicationInfo)event;
		      if (info.getMap().get("error.code") != null) {
//	                  int errorCode = Integer.parseInt((String)info.getMap().get("error.code"));
					String errorText = (String) info.getMap().get(
							"error.text");

					// Http Error 301/302 are redirects. we can try again!
//						handleRedirectError(view, event, errorText, urlBase);
					//	                  return handleApplicationError(errorCode, errorText);
  WebBrowserApp.debugOut(event+" -- app error on resource - "+errorText);
  				((WebBrowserApp)_view.getBApp()).logAppError(-1, "ABANDONING IMAGE REPLACEMENT "+_urlBase+" on APP_INFO error: "+info.getMap().get("error.code")+" "+errorText, -1);
		      }
			} else if (event.getOpCode() == HmeEvent.EVT_RSRC_INFO) {
				Map map = ((ResourceInfo) event).getMap();
WebBrowserApp.debugOut(event);					
				//If this is a resource-done or resource-error, update the view and chunk appropriately
				// and remove from queue
				// Actually, width/height can appear in different states,
				// and so can errors, we just care if we get them, not what
				// status it is.
//					if (((ResourceInfo) event).getStatus() == ResourceInfo.RSRC_STATUS_COMPLETE
//							|| ((ResourceInfo) event).getStatus() == ResourceInfo.RSRC_STATUS_READY) {
					int width = -1;
					int height = -1;
					try {
						width = Integer.parseInt(String.valueOf(map.get("width")));
						height = Integer.parseInt(String.valueOf(map.get("height")));
					} catch (NumberFormatException ex) {
					}
					if (width >= 0 && height >= 0) {
						//FIXME reduce size of too-large width
						
						//FIXME if height or width is already specified, keep it and stretch the image
						
//							imageQueue.remove(chunk.src);
						_chunk.width = width;
						_chunk.height = height;
						// update the chunk with the src that works.
						// TODO should update ALL uses of src with new one maybe use resource map to hold a redirect url.
						Resource r = ((ResourceInfo) event).getResource();
						if(r != null && r instanceof StreamResource && ((StreamResource)r).getURI() != null) {
							_chunk.src = ((StreamResource)r).getURI();
						}
						_htmlBreakdown.putResourceDetail(_chunk.src, new Dimension(width, height));
//							setSize(view, width, height);
						finishedWithResource(event);
					} else if (map.get("error.code") != null) {
//					} else if(((ResourceInfo) event).getStatus() >= ResourceInfo.RSRC_STATUS_ERROR) {
//							imageQueue.remove(chunk.src);
						// replace image with error
						String text = (String) map.get("error.text");
						// Http Error 301/302 are redirects. we can try again!
						if(!handleRedirectError(_view, event, text, _urlBase)) {
							if(!tryHTTPInstead(_view, event, text, _urlBase, _chunk)) {
							Resource errorResource;
							if(text == null) {
								errorResource = _view.getResource(IMAGE_ERROR_RESOURCE);
								_htmlBreakdown.putResourceDetail(_chunk.src, errorResource);
							} else {
								if(_view != null && _view.getResource() != null && _view.getResource() instanceof StreamResource)
									WebBrowserApp.debugOut("ERROR SRC final: "+((StreamResource)_view.getResource()).getURI());
								else
									WebBrowserApp.debugOut("ERROR SRC orig:"+_chunk.src);
								errorResource = _htmlBreakdown.createErrorTextResource(_view, text);
								_htmlBreakdown.putResourceDetail(_chunk.src, text);
							}
							
							applyNewResource(errorResource);
							
							finishedWithResource(event);
							}
						}
					} else if(((ResourceInfo) event).getStatus() >= ResourceInfo.RSRC_STATUS_COMPLETE) {
						// looks like we're not getting what we want... finish the page.
//							imageQueue.remove(chunk.src);
						width = HTMLBreakdown.DEFAULT_IMAGE_WIDTH;
						height = HTMLBreakdown.DEFAULT_IMAGE_HEIGHT;
						_chunk.width = width;
						_chunk.height = height;
//							setSize(view, width, height);
						finishedWithResource(event);
					}
					
					//FIXME oversize image gives us crappy status information...
					// closed state with no map
//						LOG: event 2575.RESOURCE_INFO(closed, com.tivo.core.ds.MapThreadWrapper@0{})
//						RSRC closed:com.tivo.core.ds.MapThreadWrapper@0{}
					// Application error about bad width, no resource id reference
//						LOG: event 1.APP_INFO(com.tivo.core.ds.MapThreadWrapper@311525890{error.code=2, error.text=max image width exceeded (1600 > 1024)})
//						APP_INFO error 2: max image width exceeded (1600 > 1024)
//						LOG: handleApplicationError(2,max image width exceeded (1600 > 1024))
					// Application error about bad height, no resource id reference
//						LOG: event 1.APP_INFO(com.tivo.core.ds.MapThreadWrapper@1128792516{error.code=2, error.text=max image height exceeded (1200 > 768)})
//						APP_INFO error 2: max image height exceeded (1200 > 768)
//						LOG: handleApplicationError(2,max image height exceeded (1200 > 768))
					// and that's IT... no more status!
//					}
			}
		}

		private boolean tryHTTPInstead(BView view, HmeEvent event,
				String text, URL referer, HTMLImageChunk chunk) {
			if(this.triedHTTP != null && this.triedHTTP.equals(chunk.src)) {
				return false;
			}
//				
//				if(true)return false;
//				
			
			if(chunk.src.startsWith("https://")) {
				this.triedHTTP = chunk.src;
				WebBrowserApp bapp = ((WebBrowserApp)view.getBApp());
				String url = bapp.connect.getFixedImageResourceSrc(chunk.src, referer);
//					String url = "http://"+chunk.src.substring("https://".length());
				WebBrowserApp.debugOut("TRYING HTTP: "+url);
				try {
					if (url != null) {
						URL resURL = new URL(url.trim());
						URL newresURL = bapp.connect.resolveRedirects(resURL, referer);
		if(!newresURL.toString().equals(url)) WebBrowserApp.debugOut("TRYING redirected HTTP: "+newresURL);
//								Resource newres = view.getResource(newresURL);
							Resource newres = _htmlBreakdown.getImageResource(bapp, newresURL.toString(), _urlBase);
							if (newres != null) {
								newres.addHandler(this);
								
								applyNewResource(newres);

								Resource res = view.getBApp().getResource(event.getID());
								if (res != null) {
									res.removeHandler(this);
								}
								return true;
							}
					}
				} catch (MalformedURLException e) {
					// give up and continue with regular error display
				}
			}
			return false;
		}

		protected void applyNewResource(Resource newres) {
			// we don't actually set the new resource.. instead continueRender will
			// do that by pulling it out of the map when it tries to establish the
			// image again.
//			view.setResource(newres);
			
//			view.setResource(errorResource,
//			ResourceInfo.RSRC_TEXT_WRAP
//			| ResourceInfo.RSRC_HALIGN_LEFT
//			| ResourceInfo.RSRC_VALIGN_TOP
//			| ResourceInfo.RSRC_IMAGE_BESTFIT);
			
		}

		private boolean handleRedirectError(BView parentView, HmeEvent rsrcInfoEvent,
				String errorText, URL urlBase) {
			// if this is a 301/302 redirect error, handle the redirect and
			// set up the new stream in the parentView - flag that we did
			// this so we don't get caught in a loop
			if(this.redirectedOnce) return false;
			if(handleRedirectError2(parentView, rsrcInfoEvent, errorText, urlBase)) {
				WebBrowserApp.debugOut("Redirect Error handled");
				this.redirectedOnce = true;
				return true;
			}
			return false;
		}

		public boolean handleRedirectError2(final BView view, HmeEvent event,
				String errorText, URL referer) {
			if (errorText != null) {
				Matcher m = ((WebBrowserApp)view.getBApp()).connect.redirectErrorPattern.matcher(errorText);
				if (m.matches()) {
					WebBrowserApp.debugOut("Matched redirect error... " + m.group(1)); //$NON-NLS-1$
					try {
						// unfortunately, this url isn't from the error, it's the
						// original one
						String url = m.group(1);
						if (url != null) {
							URL resURL = new URL(url.trim());
							URL newresURL = ((WebBrowserApp)view.getBApp()).connect.resolveRedirects(resURL, referer);
							WebBrowserApp.debugOut("REDIRECT result: "+newresURL);							
							if (!resURL.equals(newresURL)) {
								Resource newres = _htmlBreakdown.getImageResource(view.getBApp(), newresURL.toString(), referer);
//								Resource newres = view.getResource(newresURL);
								if (newres != null) {
									newres.addHandler(this);
									
									applyNewResource(newres);

									Resource res = view.getBApp().getResource(event.getID());
									if (res != null) {
										res.removeHandler(this);
									}
									return true;
								}
							}
						}
					} catch (MalformedURLException e) {
						// give up and continue with regular error display
					}
				}
			}
			return false;
		}

		protected void finishedWithResource(HmeEvent event) {
			WebBrowserApp.debugOut("finished with resource, removing handler of "+event.getID());
			// we're done with status, remove ourselves.
			Resource res = _view.getBApp().getResource(event.getID());
			if(res != null) {
				res.removeHandler(this);
			}
//				if(imageQueue.size() == 0) {
				_htmlBreakdown.continueRender();
//				}
		}
	}