package com.blackledge.david.html.hme;

import java.io.UnsupportedEncodingException;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLDecoder;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * Detects and unwraps embedly links. Embedly.com (embed.ly) is used to embed
 * other content into a webpage including youtube, for instance. Apparently to
 * enable website owners to see more stats.
 * 
 */
public class EmbedlyHandler extends VideoSourceHandler {

	public EmbedlyHandler(WebBrowserApp web) {
		super(web);
	}
	
	public boolean canHandle(URL url) {
		return EmbedlyHandler.isEmbedly(url);
	}
	
	public URL getPosterImage(URL url) {
		return EmbedlyHandler.getPoster(url);
	}
	
	public URL getVideoLink(URL url) {
		return EmbedlyHandler.getLink(url);
	}

	/**
	 * Embedly doesn't handle launching in general, but we return false here in
	 * case we end up on such a page.
	 */
	public boolean launchHasPage(URL url) {
		return false;
	}

	/**
	 * Embedly doesn't handle launching...so this just returns false.
	 */
	public boolean launchVideo(URL url, URL referer, boolean PIP) {
		return false;
	}
	
	/*
 cdn.embedly.com getting used for youtube (and other?) links - link and image (and "schema")
<iframe class="embedly-embed" 
src="//cdn.embedly.com/widgets/media.html?src=http%3A%2F%2Fwww.youtube.com%2Fembed%2FMX2XUkof1zk%3Ffeature%3Doembed&url=http%3A%2F%2Fwww.youtube.com%2Fwatch%3Fv%3DMX2XUkof1zk&image=http%3A%2F%2Fi.ytimg.com%2Fvi%2FMX2XUkof1zk%2Fhqdefault.jpg&key=bf0f5443890c4e8887da35fba9b1523a&type=text%2Fhtml&schema=youtube" 
width="500" height="375" scrolling="no" frameborder="0" allowfullscreen></iframe>
	 */
	private static final Pattern embedlyPattern = Pattern.compile("[:/a-zA-Z0-9.-]*embed(ly\\.com|\\.ly).*?src=.*");
	private static boolean isEmbedly(URL url) {
		if(url == null) return false;
		boolean result = embedlyPattern.matcher(url.toString()).matches();
//		WebBrowserApp.debugOut("EMBEDLY TEST:"+url+" "+result);
		return result;
	}
	
	private static URL getLink(URL embedlySrc) {
		return getDetail(embedlySrc, 1);
	}

	/** get the URL of an image representing the video */
	private static URL getPoster(URL embedlySrc) {
		return getDetail(embedlySrc, 2);
	}

	//FIXME making image= section optional causes it to not match at all!
	private static final Pattern embedlyDetailPattern = Pattern.compile(".*\\?.*src=([^&]+).*(?:image=([^&]+)).*");
	private static URL getDetail(URL embedlySrc, int group) {
		if(embedlySrc == null) return null;
		Matcher idMatcher = embedlyDetailPattern.matcher(embedlySrc.toString());
		if(idMatcher.find() && idMatcher.groupCount() >= group) {
			String u = idMatcher.group(group);
			try {
				u = URLDecoder.decode(u, "UTF-8");
				return new URL(u);
			} catch (MalformedURLException e) {
				// FIXME Auto-generated catch block
				e.printStackTrace();
			} catch (UnsupportedEncodingException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		WebBrowserApp.debugOut("EMBEDLY : groupCount="+idMatcher.groupCount()+" "+embedlySrc.getQuery());
		return null;
	}


	
}
