package com.blackledge.david.html.hme;

import java.awt.Color;
import java.net.URL;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

import com.blackledge.david.tivo.AppPassThroughView;
import com.tivo.hme.bananas.BApplication;
import com.tivo.hme.bananas.BText;
import com.tivo.hme.bananas.BView;
import com.tivo.hme.sdk.HmeEvent;
import com.tivo.hme.sdk.Resource;
import com.tivo.hme.sdk.HmeEvent.Key;

class VideoHandlerView extends BView {
	public static final float PIP_SCALE = 2f/7f; // I measured Premiere's pip as > 2/7ths, <.3 by width
	public static final int PIP_EDGE_OFFSET = 10; // PIP must be away from edge or scale is ignored by video.
	
	private boolean pip;
	private AppPassThroughView appPassthrough;
	private boolean showingLiveTV;
	private boolean dispatchOverride;
	private long playingUntil = -1;
	
	//TODO make API for underlying handler (appview, livetv(direct video stream), audio playlist with custom PIP handling?)
	// timeout handling - make sure it still works, make it go full screen on timeout and/or bounce around.
	//TODO "SLOW" - only livetv if option enabled currently? should it do live, or should it attempt to restore last PIP (does !visible silence it? and restore if set to visible?)?
	public VideoHandlerView(BApplication app, boolean pip, boolean visible) {
		super(app.getAbove(), 0, 0, app.getAbove().getWidth(), app.getAbove().getHeight(), visible);
		this.pip = pip;
		// background behind the real content (appPassthrough or otherwise)
		BView placeholder = new BView(this, 0, 0, getWidth(), getHeight());
		placeholder.setResource(Color.black);
		placeholder.setTransparency(.5f);
		BText hint = new BText(placeholder, 0, 0, getWidth(), getHeight());
		hint.setShadow(true);
		hint.setColor(Color.white);
		hint.setFont("default-50-bold.font");
		hint.setValue("Zoom / Aspect:\n" +
				"\n" +
				"Toggle Full Screen");
	}
	
	public boolean keyDispatchOverride(Key ir) {
		if (this.dispatchOverride) {
			getResourceParent().postEvent(ir);
			return true;
		} else
			return false;
	}

	private BView getResourceParent() {
		return this.appPassthrough;
	}
	
	public void pause() {
		if (this.appPassthrough != null
				&& this.appPassthrough.getResource() != null) {
			HmeEvent pauseEvent = new HmeEvent.Key(1, KEY_PRESS,
					KEY_PAUSE, 0);
			this.appPassthrough.postEvent(pauseEvent);
			pauseEvent = new HmeEvent.Key(1, KEY_RELEASE,
					KEY_PAUSE, 0);
			this.appPassthrough.postEvent(pauseEvent);
		}
	}

	public void close() {
		this.showingLiveTV = false;
		this.playingUntil = -1;
		allowIdleTimeout(true);
		if(this.appPassthrough != null) {
			if(this.appPassthrough.getResource() != null) {
				this.appPassthrough.getResource().remove();
			}
			this.appPassthrough.remove();
		}
		this.setVisible(false);
	}

	public void startAppVideo(String hmeURL, Map hmeMap, boolean startPIP) {
		this.setVisible(true);
		this.setTransparency(0.0f, getResource("*200"));
		
		if(this.appPassthrough != null) {
			this.showingLiveTV = false;// if this was called TO show livetv, this variable gets set afterwards.
			if(this.appPassthrough.getResource() != null) {
				this.appPassthrough.getResource().remove();
			}
			this.appPassthrough.remove();
		}
		this.appPassthrough = new AppPassThroughView(this, 0, 0, this.getWidth(), this.getHeight());
		this.pip = startPIP;
		setupPIP(this.pip);
		this.appPassthrough.setAppActions("app_started", "app_ended");
		this.playingUntil = 0; // indefinite play length.
		this.appPassthrough.startApp(hmeURL, hmeMap);
	}
	
	private void allowIdleTimeout(boolean allow) {
		((WebBrowserApp)getBApp()).setAllowIdleTimeout(allow);
	}
	
	public void startLiveTV() {
		String videoUrl = WebBrowserApp.STREAM_LIVETV;
		String title = "Live TV";
		
		try {
			URL url = new URL("hme://204.176.49.116:7288/videoplayer?" //$NON-NLS-1$
					+ "title="+title //$NON-NLS-1$
//					+ "&" //$NON-NLS-1$
//					+ "subtitle="+subtitle //$NON-NLS-1$
//					+ "&" //$NON-NLS-1$
//					+ "description="+description //$NON-NLS-1$
					+ "&" //$NON-NLS-1$
					+ "uri="+URLEncoder.encode(videoUrl, "UTF-8") //$NON-NLS-1$ //$NON-NLS-2$
//					+ durationString
					);
			this.showingLiveTV = true;
			startAppVideo(((WebBrowserApp) getBApp()).connect.getHmeURL(url), ((WebBrowserApp)getBApp()).getHmeMap(url), true);
			this.showingLiveTV = true;
		} catch(Exception e) {
			getBApp().play(WebBrowserApp.SOUND_BONK);
			// TODO no live tv for you!
		}
	}
	
	public boolean isShowingLiveTV() {
		return showingLiveTV;
	}
	
	public boolean isPip() {
		return pip;
	}
	
	public void setPip(boolean pip) {
		boolean changed = pip != this.pip;
		this.pip = pip;
		if(changed) {
			setupPIP(this.pip);
		}
	}
	
	private void setupPIP(boolean pip) {
		if(this.appPassthrough != null) {
			List<Integer> handlerkeys = new ArrayList<Integer>();
			handlerkeys.add(KEY_OPT_PIP);//Zoom/Aspect/Window/PIP
			if(pip) {
				this.dispatchOverride = false;
			} else {
				handlerkeys.add(KEY_LEFT);// activate PIP
				this.dispatchOverride = true;
			}
			this.appPassthrough.setKeyPressOverrides(handlerkeys, this);
			if (this.appPassthrough.isStarted()) {
				scalePIP(pip);
			}
		}
	}
	
	/** Move the video back after a hide() */
	public void show() {
		allowIdleTimeout(false);
		show(getResource(
//				null));
				"*500"));
		setVisible(true);
	}
	
	private void show(Resource anim) {
		if(this.pip) {
			float pipScale = PIP_SCALE;
			int pipWidth = (int) (this.getParent().getWidth() * pipScale);
			int edgeOffset = PIP_EDGE_OFFSET;
			this.setLocation(this.getParent().getWidth()-pipWidth-edgeOffset, edgeOffset, anim);
		}
	}
	
	/**
	 * Move the video out of the way temporarily (still visible if no other
	 * content covers the same area)
	 */
	public void hide() {
		allowIdleTimeout(true);
		if(this.pip) {
//			this.setVisible(false);
			int edgeOffset = PIP_EDGE_OFFSET;
			this.setLocation(this.getParent().getWidth()-edgeOffset, edgeOffset, getResource("*500"));
		}
	}

	private void scalePIP(boolean pip) {
		synchronized(this) {
			boolean wasPainting = this.getPainting();
			this.setPainting(false);
			if (pip) {
				float pipScale = PIP_SCALE;
				show(null);
				flush(); try { Thread.sleep(100); } catch(Exception e) {}
				this.setScale(pipScale, pipScale);
			} else {
				this.setLocation(0, 0);
				flush(); try { Thread.sleep(100); } catch(Exception e) {}
				this.setScale(1.0f, 1.0f);
			}
			if (wasPainting) {
				this.setPainting(true);
			}
		}
	}
	
	@Override
	public boolean handleAction(BView view, Object action) {
		if(action.equals("app_started")) {
			allowIdleTimeout(false);
			this.playingUntil = 0; // active for an indefinite time.
			scalePIP(this.pip);
		} else if(action.equals("app_ended")) {
			this.showingLiveTV = false;
			allowIdleTimeout(true);
			int hideMillis = 500;
			this.playingUntil = new Date().getTime() + hideMillis + 200; // active until fade is done (with some buffer time for network)
			Resource anim = this.getResource("*" + hideMillis);
			this.setTransparency(1.0f, anim);
			this.setVisible(false, anim);
			// restore control
			this.dispatchOverride = false;
		}
		
		return super.handleAction(view, action);
	}
	
	@Override
	public boolean handleKeyPress(int code, long rawcode) {
		switch(code) {
		case KEY_OPT_PIP://Zoom/Aspect/Window/PIP
			setPip(!isPip());
			return true;
		case KEY_LEFT:
			setPip(true);
			return true;
		}
		return true;
//		return super.handleKeyPress(code, rawcode);
	}

	public boolean isPlaying() {
		if (playingUntil > 0) {
			if (new Date().getTime() > playingUntil) {
				playingUntil = -1;
				return false;
			} else {
				return true;
			}
		} else {
			return playingUntil == 0;
		}
	}
}