package com.blackledge.david.html.hme;

import java.awt.Color;
import java.net.MalformedURLException;
import java.net.URL;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;
import java.util.regex.Matcher;

import com.tivo.core.ds.TeDict;
import com.tivo.hme.bananas.BEvent;
import com.tivo.hme.bananas.BView;
import com.tivo.hme.sdk.HmeEvent;
import com.tivo.hme.sdk.IHmeEventHandler;
import com.tivo.hme.sdk.Resource;
import com.tivo.hme.sdk.HmeEvent.ResourceInfo;
import com.tivo.hme.sdk.StreamResource;
import com.tivo.hme.sdk.View;

public class VideoWatcher extends BView implements IHmeEventHandler {

	private static final String RES_MAP_KEY_ERROR_TEXT = "error.text"; //$NON-NLS-1$
	private static final String RES_MAP_KEY_ERROR_CODE = "error.code"; //$NON-NLS-1$
	private static final int HEIGHT = 100;
//	private static final int WIDTH = 400;
	private StreamResource videoResource;
	private BView resourceParent;
	private String contentType;
    protected static final float speed_forward3 = 60.0f;
    protected static final float speed_forward2 = 18.0f;
    protected static final float speed_forward1 = 3.0f;
    protected static final float speed_slow = 0.25f;
    protected static final float speed_normal = 1.0f;
    protected static final float speed_paused = 0.0f;
    protected static final float speed_reverse1 = -3.0f;
    protected static final float speed_reverse2 = -18.0f;
    protected static final float speed_reverse3 = -60.0f;
    
	private static final float SCALE_NORMAL_Y = 1.0f;
	private static final float SCALE_NORMAL_X = 1.0f;
	private static final float SCALE_LETTERBOX_X = 1.0f;
	/**
	 * We want letterbox because a 16x9 was squashed to a 4x3, and we want to
	 * further squash it so it's like 16x9 again. x squashed by 3/4, so squash y
	 * by 3/4
	 */
	private static final float SCALE_LETTERBOX_Y = 0.75f;	
	/**
	 * We want pillarbox because a 4x3 was stretched to a 16x9 and we want to
	 * squash it back to 4x3. x stretched by 4/3, so squash x by 3/4
	 */
	private static final float SCALE_PILLARBOX_X = 0.75f;
	private static final float SCALE_PILLARBOX_Y = 1.0f;
    
	private StreamResource currentStream;
    protected ArrayList<String> streamLinks;
    protected ArrayList<Long> streamLengths;
    protected ArrayList<String> streamTitles;
	private String currentTitle;
	private Long currentLength;

	private String start_action;
	private String next_action;
	private String end_action;
	private Object font;
	private Object color;
	private URL referer;

	public void setActions(String start, String next, String end) {
		start_action = start;
		next_action = next;
		end_action = end;
	}
	
	public VideoWatcher(BView parent, BView resourceParent, String contentType) {
		super(parent, 0, 0, parent.getWidth(), parent.getHeight());
//		this.setFocusable(true); // not necessary
		this.resourceParent = resourceParent;
		this.contentType = contentType;
		// use default font settings
		setTextFont(null, null);
	}
	
	@Override
	public boolean handleKeyPress(int code, long rawcode) {
		if(videoResource != null) {
			switch(code) {
			case KEY_OPT_ASPECT://Zoom/Aspect/Window/PIP
				toggleAspect();
				return true;
			case KEY_PLAY:
				videoResource.play();
				return true;
			case KEY_PAUSE:
				// Automatically unpauses if paused
				videoResource.pause();
				return true;
			case KEY_LEFT:
				this.remove();
				getBApp().pop();
				return true;
			case KEY_FORWARD:
				if(contentType != null && contentType.startsWith("video/")) { //$NON-NLS-1$
					switch((int)videoResource.getSpeed()) {
					case (int) speed_reverse3:
						play("speedup2.snd"); //$NON-NLS-1$
						videoResource.setSpeed(speed_reverse2);
					break;
					case (int) speed_reverse2:
						play("speedup1.snd"); //$NON-NLS-1$
						videoResource.setSpeed(speed_reverse1);
					break;
					case (int) speed_normal:
						play("speedup1.snd"); //$NON-NLS-1$
						videoResource.setSpeed(speed_forward1);
					break;
					case (int) speed_forward1:
						play("speedup2.snd"); //$NON-NLS-1$
						videoResource.setSpeed(speed_forward2);
					break;
					case (int) speed_forward2:
						play("speedup3.snd"); //$NON-NLS-1$
						videoResource.setSpeed(speed_forward3);
					break;
					case (int) speed_paused:
						// do nothing;
						break;
					default:
						play("slowdown1.snd"); //$NON-NLS-1$
						videoResource.setSpeed(speed_normal);
					break;
					}
					return true;
				}
				return false;
			case KEY_REVERSE:
				if(contentType != null && contentType.startsWith("video/")) { //$NON-NLS-1$
					switch((int)videoResource.getSpeed()) {
					case (int) speed_reverse2:
						play("speedup3.snd"); //$NON-NLS-1$
						videoResource.setSpeed(speed_reverse3);
					break;
					case (int) speed_reverse1:
						play("speedup2.snd"); //$NON-NLS-1$
						videoResource.setSpeed(speed_reverse2);
					break;
					case (int) speed_normal:
						play("speedup1.snd"); //$NON-NLS-1$
						videoResource.setSpeed(speed_reverse1);
					break;
					case (int) speed_forward2:
						play("speedup1.snd"); //$NON-NLS-1$
						videoResource.setSpeed(speed_forward1);
					break;
					case (int) speed_forward3:
						play("speedup2.snd"); //$NON-NLS-1$
						videoResource.setSpeed(speed_forward2);
					break;
					case (int) speed_paused:
						// do nothing;
						//FIXME pop?
						break;
					default:
						play("slowdown1.snd"); //$NON-NLS-1$
						videoResource.setSpeed(speed_normal);
					break;
					}
				} else {
					this.remove();
					getBApp().pop();
				}
				return true;
			default:
				// eat all other events.
				return true;
			}
		} else return true;
//		return super.handleKeyPress(code, rawcode);
	}

	private void toggleAspect() {
		View resourceOwner = getOwner(getParent().getParent(), videoResource);
		if(resourceOwner != null) {
			float x = resourceOwner.getScaleX();
			float y = resourceOwner.getScaleY();
			float fullWidth = getBApp().getNormal().getWidth();
			float fullHeight = getBApp().getNormal().getHeight();
			float actionWidth = fullWidth - 2f*getBApp().getSafeActionHorizontal();
			float actionHeight = fullHeight - 2f*getBApp().getSafeActionVertical();
			float actionScaleX = actionWidth/fullWidth;
			float actionScaleY = actionHeight/fullHeight;
			
			// scale greater than 1.0 doesn't work.
			// switch between 1.0x1.0 and something else flashes screen (probably switch to/from a "fullscreen mode")
			float xs[] = new float[] {
					SCALE_NORMAL_X,
					SCALE_NORMAL_X*actionScaleX,
					SCALE_LETTERBOX_X,
					SCALE_LETTERBOX_X*actionScaleX,
					SCALE_PILLARBOX_X,
					SCALE_PILLARBOX_X*actionScaleX,
			};
			float ys[] = new float[] {
					SCALE_NORMAL_Y,
					SCALE_NORMAL_Y*actionScaleY,
					SCALE_LETTERBOX_Y,
					SCALE_LETTERBOX_Y*actionScaleY,
					SCALE_PILLARBOX_Y,
					SCALE_PILLARBOX_Y*actionScaleY,
			};
			for(int i = 0 ; i < xs.length ; ++i) {
				if(x == xs[i] && y == ys[i]) {
					if(i+1 < xs.length) {
						scale(resourceOwner, xs[i+1], ys[i+1]);
					} else {
						scale(resourceOwner, xs[0], ys[0]);
					}
					break;
				}
			}
		}
	}

	private void scale(View resourceOwner, float scaleX, float scaleY) {
		resourceOwner.setScale(scaleX, scaleY);
		double heightDiff = resourceOwner.getHeight()-(resourceOwner.getHeight()*scaleY);
		double widthDiff = resourceOwner.getWidth()-(resourceOwner.getWidth()*scaleX);
		resourceOwner.setLocation(resourceOwner.getX(), resourceOwner.getY());
		resourceOwner.setSize(resourceOwner.getWidth(), resourceOwner.getHeight());
		resourceOwner.getParent().setTranslation((int) (widthDiff/2), (int) (heightDiff/2));
	}
	
	private View getOwner(View parent, Resource resource) {
		for(int i = 0 ; i < parent.getChildCount() ; ++i) {
			View child = parent.getChild(i);
			if(child.getResource() == resource) {
				return child;
			} else {
				View owner = getOwner(child, resource);
				if(owner != null)
					return owner;
			}
		}
		return null;
	}

	@Override
	public void remove(Resource animation) {
		if(videoResource != null) {
			videoResource.close();
			videoResource.remove();
			videoResource = null;
		}
		super.remove(animation);
	}
	
	@Override
    public boolean handleEvent(HmeEvent event) {
		if(event instanceof BEvent.Focus) {
			if(((BEvent.Focus) event).getLost() == this) {
				if(videoResource != null) {
					videoResource.close();
					videoResource.remove();
					videoResource = null;
				}
			}
		}
        if(event instanceof HmeEvent.ResourceInfo) {
            ResourceInfo resInf = (ResourceInfo) event;
//debugOut(resInf);
			if (resInf.getResource() != null
					&& resInf.getResource() instanceof StreamResource) {
				videoResource = (StreamResource) resInf.getResource();
			}
            // Status changes only
            if(resInf.getOpCode() == Resource.EVT_RSRC_STATUS) {
            	int videoStatus = resInf.getStatus();
            	switch(videoStatus) {
            	case RSRC_STATUS_CONNECTING:
                    setText(Messages.getString("VideoWatcher.status_connecting"), null); //$NON-NLS-1$
            		break;
            	case RSRC_STATUS_CONNECTED:
                    setText(Messages.getString("VideoWatcher.status_connected"), null); //$NON-NLS-1$
            		break;
            	case RSRC_STATUS_LOADING:
                    setText(Messages.getString("VideoWatcher.status_loading"), null); //$NON-NLS-1$
            		break;
            	case RSRC_STATUS_READY:
                    setText(Messages.getString("VideoWatcher.status_ready"), "*1000,-1.0"); //$NON-NLS-1$ //$NON-NLS-2$
            		break;
            	case RSRC_STATUS_PLAYING:
                    setText(Messages.getString("VideoWatcher.status_playing"), "*1000"); //$NON-NLS-1$ //$NON-NLS-2$
            		break;
            	case RSRC_STATUS_PAUSED:
                    setText(Messages.getString("VideoWatcher.status_paused"), "*5000,0.5"); //$NON-NLS-1$ //$NON-NLS-2$
            		break;
            	case RSRC_STATUS_SEEKING:
                    setText(Messages.getString("VideoWatcher.status_seeking"), null); //$NON-NLS-1$
            		break;
            	case RSRC_STATUS_CLOSED:
                    setText(Messages.getString("VideoWatcher.status_closed"), null); //$NON-NLS-1$
                    break;
                    
        		// COMPLETE means it finished downloading, regardless of whether
        		// it is finished playing.
            	case RSRC_STATUS_COMPLETE:
            		// Complete replaces Playing when we're done downloading.
            		// if we're stopped due to buffer_overflow, we should auto-restart
            		// FIXME however, what if they manually paused before it completed? no good!
            		// FIXME check this: it's possible it uses PLAYING again if you pause and restart after complete
//	            		videoResource.play();
            		
					// audio streams end with "COMPLETE" while video just
					// finished downloading with "COMPLETE" and ends with
					// "END"
            		if(this instanceof AudioWatcher) {
	                    setText(Messages.getString("VideoWatcher.status_complete"), null); //$NON-NLS-1$
	                    StreamResource nextStream = startNextStream();
						if(nextStream == null) {
							freeTimeout();
						} else {
							resourceParent.setResource(nextStream);
						}
            		} else {
            			//Video will just clear status and keep clearing it until something new happens
            			setText(null, null);
            		}
            		break;

            	case RSRC_STATUS_ERROR:
            		String errorMess = getError(resInf);
            		setText(errorMess, null);
            		// don't write any other status to the text after this
            		resInf.getResource().removeHandler(this);
            		
                    StreamResource nextStreamErr = startNextStream();
					if(nextStreamErr == null) {
						freeTimeout();
						if(end_action != null) {
							postEvent(new BEvent.Action(this, end_action));
						}
					} else {
						resourceParent.setResource(nextStreamErr);
					}
            		break;
            		
            	case RSRC_STATUS_END:
                    setText(Messages.getString("VideoWatcher.status_end"), null); //$NON-NLS-1$
                    StreamResource nextStream = startNextStream();
					if(nextStream == null) {
						freeTimeout();
					} else {
						resourceParent.setResource(nextStream);
					}
            		break;
            		
            	case RSRC_STATUS_BUFFER_OVERFLOW:
                    setText(Messages.getString("VideoWatcher.status_buffer_end"), null); //$NON-NLS-1$
            		break;
            	}
            	return false; // allow status to bubble up.
            } else if(resInf.getOpCode() == Resource.EVT_RSRC_INFO) {
            	// handle a known flaw... TiVo returns (http) error code and text in a regular "connected" message.
            	if((resInf.getStatus() != Resource.RSRC_STATUS_ERROR) && resInf.getMap() != null) {
            		String code = (String) resInf.getMap().get(RES_MAP_KEY_ERROR_CODE);
            		if(code != null) {
						// don't let a follow-on error status overwrite the text with nothing useful.
						resInf.getResource().removeHandler(this);
            			// see if the problem is redirects and get a replacement url.
            			URL newURL = handleRedirects(resInf);
            			if(newURL != null) {
            				StreamResource stream = startStream( newURL.toString(), currentTitle, currentLength, contentType);
							resourceParent.setResource(stream);
            			} else {
            				// default error behavior
							String errorMess = getError(resInf);
							setText(errorMess, null);
							
                            StreamResource nextStream = startNextStream();
        					if(nextStream == null) {
        						freeTimeout();
    							if(end_action != null) {
    								postEvent(new BEvent.Action(this, end_action));
    							}
        					} else {
        						resourceParent.setResource(nextStream);
        					}
            			}
            		}
            	}
            	return false; // allow info to bubble up
            }
        }
        //NOTE: have to call this to do ACTION, FOCUS, and KEY events properly
        return super.handleEvent( event );
    }
	
	public StreamResource startNextStream() {
		if(streamLinks == null || streamLinks.size() == 0) {
			if(end_action != null) {
				postEvent(new BEvent.Action(this, end_action));
			}
			return null;
		}
		String stream = streamLinks.remove(0);
		String title = streamTitles.remove(0);
		Long length = streamLengths.remove(0);
		return startStream(stream, title, length, null);
	}

	private StreamResource startStream(String stream, String title, Long length, String theContentType) {
		WebBrowserApp wapp = ((WebBrowserApp)getBApp());
		this.lastText = null;
		boolean starting = (currentStream == null);
		if(currentStream != null) {
			currentStream.removeHandler(this);
			currentStream.remove();
			currentStream = null;
		}
		while (stream != null
				&& (stream.toLowerCase().endsWith(".pls") //$NON-NLS-1$
						|| stream.toLowerCase().endsWith(".m3u") || stream //$NON-NLS-1$
						.toLowerCase().endsWith(".m3u8"))) { //$NON-NLS-1$
			
			URL url;
			try {
				url = new URL(
						//FIXME add urlBase, (use this.referer?)
						referer,
						stream);
			} catch (MalformedURLException e) {
				// try the next stream in the list
				if(streamLinks.size() > 0) {
					stream = streamLinks.remove(0);
					title = streamTitles.remove(0);
					length = streamLengths.remove(0);
				} else {
					stream = null;
					title = null;
					length = null;
				}
				continue;
			}
			String file = wapp.connect.getRawText(url, referer);
//			debugOut("File "+stream+"\n"+file);
			if(file == null) {
				if(end_action != null) {
					postEvent(new BEvent.Action(this, end_action));
				}
				return null;
			}
			if(file.trim().startsWith("[playlist]")) //$NON-NLS-1$
				this.insertPLS(file);
			else
				this.insertM3U(file);
			
			// get the first stream definition that was just inserted.
			if(streamLinks.size() > 0) {
				stream = streamLinks.remove(0);
				title = streamTitles.remove(0);
				length = streamLengths.remove(0);
			} else {
				stream = null;
				title = null;
				length = null;
			}
		}
debugOut("start stream "+stream); //$NON-NLS-1$
// give a hint on the stream... 
//FIXME at least check the suffix!
		if(theContentType == null) {
			theContentType = "video/mpeg2"; //$NON-NLS-1$
			if(this instanceof AudioWatcher)
				theContentType = "audio/mpeg"; //$NON-NLS-1$
		}
		currentTitle = title;
		if(currentTitle == null) {
			currentTitle = stream;
			if(currentTitle != null) {
				int idx = currentTitle.lastIndexOf('/');
				if(idx > 0) {
					currentTitle = currentTitle.substring(idx);
				}
			}
		}
		currentLength = length;
		TeDict dict = null;
		if(currentLength != null) {
			dict = new TeDict();
			// lengths are in seconds, Tivo Duration is in millis
			long duration = currentLength*1000;
			dict.add(WebBrowserApp.HTTP_HEADER_TIVO_DURATION, duration);
			dict.add("TiVo-Accurate-Duration", duration); //$NON-NLS-1$
			dict.add("Accurate-Duration", duration); //$NON-NLS-1$
			dict.add("Duration", duration); //$NON-NLS-1$
debugOut("dict setting "+WebBrowserApp.HTTP_HEADER_TIVO_DURATION+": "+duration); //$NON-NLS-1$
		}
		if(starting) {
			if(start_action != null) {
				postEvent(new BEvent.Action(this, start_action));
			}
		} else {
			if(next_action != null) {
				postEvent(new BEvent.Action(this, next_action));
			}
		}
		
		//TiVo can't handle https in resources
		if (theContentType != null && theContentType.startsWith("audio/")) {
			stream = wapp.connect.getFixedAudioResourceSrc(stream, referer);
		} else {
			stream = wapp.connect.getFixedVideoResourceSrc(stream, referer);
		}
		
		currentStream = createStream(stream, theContentType, true, dict);
		currentStream.addHandler(this);
		if(currentStream == null) {
			if(end_action != null) {
				postEvent(new BEvent.Action(this, end_action));
			}
		}
		return currentStream;
	}

	public int setM3U(String file, URL referer) {
		if(file == null || file.trim().length() == 0) return 0;
		this.referer = referer;
		this.streamLengths = new ArrayList<Long>();
		this.streamTitles = new ArrayList<String>();
		ArrayList<String> links = getM3ULinks(file, this.streamLengths, this.streamTitles);
		this.streamLinks = links;
		return this.streamLinks.size();
	}

	public int insertM3U(String file) {
		if(file == null || file.trim().length() == 0) return this.streamLinks.size();
		ArrayList<String> titles = new ArrayList<String>();
		ArrayList<Long> lengths = new ArrayList<Long>();
		ArrayList<String> links = getM3ULinks(file, lengths, titles);
		this.streamLinks.addAll(0, links);
		this.streamTitles.addAll(0, titles);
		this.streamLengths.addAll(0, lengths);
		return this.streamLinks.size();
	}

	private ArrayList<String> getM3ULinks(String file, ArrayList<Long> lengths, ArrayList<String> titles) {
		ArrayList<String> links = new ArrayList<String>();
		if (file.trim().length() > 0) {
			String[] lines = file.split("\n"); //$NON-NLS-1$
			String title = null;
			Long length = null;
			for (String line : lines) {
				if (line.trim().length() > 0 && !line.startsWith("#")) { //$NON-NLS-1$
					links.add(line);
					lengths.add(length);
					titles.add(title);
					length = null;
					title = null;
				}
				if(line.startsWith("#EXTINF")) { //$NON-NLS-1$
					int comma = line.indexOf(","); //$NON-NLS-1$
					if(comma < 0)
						comma = line.length();
					String lengthString = line.substring("#EXTINF".length(), comma).trim(); //$NON-NLS-1$
					length = null;
					// trim any other chars before length
					while (lengthString.length() > 0
							&& !Character.isDigit(lengthString.charAt(0))
							&& lengthString.charAt(0) != '-') {
						lengthString = lengthString.substring(1).trim();
					}
					if (lengthString.length() > 0) {
						try {
							length = new Long(lengthString);
						} catch (Exception ex) {
							length = null;
						}
					}
					title = null;
					if(comma+1 < line.length()) {
						title = line.substring(comma+1).trim();
					}
				}
			}
		}
		WebBrowserApp.debugOut("M3U links:\n"+links); //$NON-NLS-1$
		return links;
	}

	public int setPLS(String file, URL referer) {
		if(file == null || file.trim().length() == 0) return 0;
		this.referer = referer;
		this.streamLengths = new ArrayList<Long>();
		this.streamTitles = new ArrayList<String>();
		ArrayList<String> links = getPLSLinks(file, this.streamLengths, this.streamTitles);
		this.streamLinks = links;
		return this.streamLinks.size();
	}

	public int insertPLS(String file) {
		if(file == null || file.trim().length() == 0) return this.streamLinks.size();
		ArrayList<Long> lengths = new ArrayList<Long>();
		ArrayList<String> titles = new ArrayList<String>();
		ArrayList<String> links = getPLSLinks(file, lengths, titles);
		this.streamLinks.addAll(0, links);
		this.streamLengths.addAll(0, lengths);
		this.streamTitles.addAll(0, titles);
		return this.streamLinks.size();
	}

	private ArrayList<String> getPLSLinks(String file, ArrayList<Long> lengths, ArrayList<String> titles) {
		//		[playlist]
		//		File1=URL
		//		Title1=Title
		//		Length1=secs
		//		NumberOfEntries=maxEntryNum
		//		Version=2
		HashMap<String, String> pls = new HashMap<String, String>();
		if (file.trim().length() > 0) {
			String[] lines = file.split("\n"); //$NON-NLS-1$
			for (String line : lines) {
				int eq = line.indexOf("="); //$NON-NLS-1$
				if (eq > 0) {
					pls.put(line.substring(0, eq), line.substring(eq + 1));
				}
			}
		}
		ArrayList<String> links = new ArrayList<String>();
		int maxLink = 100;
		try {
			maxLink = Integer.parseInt( pls.get("NumberOfEntries")); //$NON-NLS-1$
		} catch (Exception ex) {
			// whatever.
		}
		for(int i = 1 ; i <= maxLink ; ++i) {
			String val = pls.get("File"+i); //$NON-NLS-1$
			String title = pls.get("Title"+i); //$NON-NLS-1$
			if(title != null) {
				title = title.trim();
			}
			String lengthString = pls.get("Length"+i); //$NON-NLS-1$
			Long length = null;
			if(lengthString != null && lengthString.trim().length() > 0) {
				length = new Long(lengthString.trim());
			}
			//TODO make use of length and title
			if(val != null) {
				links.add(val);
				lengths.add(length);
				titles.add(title);
			}
		}
WebBrowserApp.debugOut("PLS links:\n"+links); //$NON-NLS-1$
		return links;
	}
	
	/**
	 * Not using JSoup because we don't need cookies - TiVo won't send them, so we shouldn't need them either.
	 * @param resInf
	 * @return
	 */
	private URL handleRedirects(ResourceInfo resInf) {
//		int errorCode=-1;
//		try {
//			errorCode = Integer.parseInt((String) resInf.getMap().get(RES_MAP_KEY_ERROR_CODE));
//		} catch(Exception e1) {
//			return null;
//		}
		String errorText=""; //$NON-NLS-1$
		errorText = (String) resInf.getMap().get(RES_MAP_KEY_ERROR_TEXT);
		if(errorText != null) {
			if (errorText != null) {
				Matcher m = ((WebBrowserApp)getBApp()).connect.redirectErrorPattern.matcher(errorText);
				if (m.matches()) {
//	debugOut("Matched redirect error... "+m.group(1));
					try {
						//unfortunately, this url isn't from the error, it's the original one
						String url = m.group(1);
						if (url != null) {
							URL resURL = new URL(url.trim());
							URL newresURL = ((WebBrowserApp)getBApp()).connect.resolveRedirects(resURL, null);
							if(!resURL.equals(newresURL)) {
								return newresURL;
							}
						}
					} catch(Exception e) {
						debugOut("...redirect attempt had error: "+e);
						// oh well
						return null;
					}
				}
			}
		}
		return null;
	}

	private String getError(ResourceInfo resInf) {
		String errorMess = ""; //$NON-NLS-1$
		Map resMap = resInf.getMap();
		if(resMap != null) {
		    // don't overwrite error info on next event.
		    String erCod = (String)resMap.get(RES_MAP_KEY_ERROR_CODE);
		    if(erCod != null)
		        errorMess += getResourceErrorCodeText(erCod);
		    String erMes = (String)resMap.get(RES_MAP_KEY_ERROR_TEXT);
		    if(erMes != null)
		        errorMess += "\n"+ erMes; //$NON-NLS-1$
//                        videoDuration = getLong((String)resMap.get("duration"));
//                        videoPosition = getLong((String)resMap.get("position"));
//                        // pos is "duration/position"
//                        videoStart = getLong((String)resMap.get("start"));
//                        videoSpeed = getDouble((String)resMap.get("speed"));
//                        videoSpan = (String)resMap.get("span"); 
//                        // span is start/end of buffered span, but at a 1/1000 scale
		}
		if(errorMess.trim().length() == 0) {
			errorMess = Messages.getString("VideoWatcher.error_unspecified"); //$NON-NLS-1$
		}
debugOut(resInf+": "+errorMess); //$NON-NLS-1$
		return errorMess;
	}

	private void debugOut(Object mess) {
		WebBrowserApp.debugOut(mess);
	}

	/** our WebScreen prevents idle timeouts when we're set up, we want to allow them again when this is called */
	private void freeTimeout() {
		((WebBrowserApp)getBApp()).setAllowIdleTimeout(true);
	}

	public void setTextFont(Object font, Object color) {
		this.font = font;
		this.color = color;
		if(this.font == null) {
			this.font = "default-20.font"; //$NON-NLS-1$
		}
		if(this.color == null) {
			this.color = Color.white;
		}
	}
	
	private String lastText = null;
    protected void setText(String text, String hideAnim) {
    	if(text == null) {
    		setVisible(false);
		} else {
    		String of = "";
    		if(streamLinks != null && streamLinks.size() > 0) {
    			of = "("+streamLinks.size()+" more)";
    		}
    		if(currentTitle != null) {
    			text = text.trim()+of+": "+currentTitle.trim(); //$NON-NLS-1$
    		} else {
    			text = text.trim()+of;
    		}
    		if (!text.equals(lastText)) {    		
    			setVisible(true);
    			setTransparency(0.0f);
    			if(getResource() != null) getResource().remove();
    			setResource(createText(this.font,this.color, text),
    					RSRC_VALIGN_CENTER|RSRC_HALIGN_LEFT);
    			if(hideAnim != null) {
    				setTransparency(1.0f, getResource(hideAnim));
    			}
    		}
    	}
    	lastText = text;
	}
    
    /** May be temporarily null.  Includes title if not null */
    public String getCurrentStatusText() {
    	return lastText;
    }
    
    /** May be null */
    public String getCurrentTitle() {
    	return currentTitle;
    }

	public static String getResourceErrorCodeText(String errorCodeNum) {
        if(errorCodeNum != null) {
              try {
                  int erCode = Integer.parseInt(errorCodeNum);
                  switch(erCode) {
                  case RSRC_ERROR_BAD_ARGUMENT:
                  case RSRC_ERROR_BAD_DATA:
                  case RSRC_ERROR_BAD_MAGIC:
                  case RSRC_ERROR_BAD_STATE:
                  case RSRC_ERROR_BAD_VERSION:
                      errorCodeNum = MessageFormat.format(Messages.getString("VideoWatcher.error_catchall"),erCode); //$NON-NLS-1$
                      break;
                  case RSRC_ERROR_CONNECT_FAILED:
                      errorCodeNum = Messages.getString("VideoWatcher.error_connect_failed"); //$NON-NLS-1$
                      break;
                  case RSRC_ERROR_CONNECTION_LOST:
                      errorCodeNum = Messages.getString("VideoWatcher.error_connection_lost"); //$NON-NLS-1$
                      break;
                  case RSRC_ERROR_CONNECTION_TIMEOUT:
                      errorCodeNum = Messages.getString("VideoWatcher.error_connection_timeout"); //$NON-NLS-1$
                      break;
                  case RSRC_ERROR_HOST_NOT_FOUND:
                      errorCodeNum = Messages.getString("VideoWatcher.error_host_not_found"); //$NON-NLS-1$
                      break;
                  case RSRC_ERROR_INCOMPATIBLE:
                      errorCodeNum = Messages.getString("VideoWatcher.error_incompatible"); //$NON-NLS-1$
                      break;
                  case RSRC_ERROR_NOT_SUPPORTED:
                      errorCodeNum = Messages.getString("VideoWatcher.error_not_supported"); //$NON-NLS-1$
                      break;
                  case RSRC_ERROR_OTHER:
                      errorCodeNum = Messages.getString("VideoWatcher.error_other"); //$NON-NLS-1$
                      break;
                  case RSRC_ERROR_UNKNOWN:
                      errorCodeNum = Messages.getString("VideoWatcher.error_unknown"); //$NON-NLS-1$
                      break;
                  }
              } catch(NumberFormatException e) {
              }
              //TODO getting errorCode 11 with 404 HTTP errors...maybe 11 = http error.
          }
        return errorCodeNum;
    }

    /** Media status code, resource at end of stream. */
    public static final int RSRC_STATUS_END = 11;
    /** Media status code, resource at end of available buffer. */
    public static final int RSRC_STATUS_BUFFER_OVERFLOW = 12;

}
