package com.blackledge.david.html.hme.host;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.net.HttpCookie;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Date;
import java.util.LinkedHashMap;
import java.util.Vector;

import com.blackledge.david.html.hme.Advertisement;
import com.blackledge.david.html.hme.Bookmarks;
import com.blackledge.david.html.hme.WebBrowserApp;

import com.blackledge.david.tivo.V49IHmeProtocol;

import com.tivo.hme.host.http.server.HttpRequest;
import com.tivo.hme.host.http.share.Query;
//import com.tivo.hme.host.sample.Listener;
import com.tivo.hme.host.util.Config;
import com.tivo.hme.interfaces.IFactory;
import com.tivo.hme.interfaces.ILogger;

import com.blackledge.david.html.hme.host.Listener;

public class WebBrowserTiVoConnect extends TiVoConnect {

	public WebBrowserTiVoConnect(Config config, Listener applistener,
			ILogger logger) throws IOException {
		super(config, applistener, logger);
	}

	@Override
	protected String getServerTitle() {
		return "Enter Webz Server";
	}

	@Override
	protected HMEItem[] getHMEItems(HttpRequest request) {
//		long start1 = new Date().getTime();

		ArrayList<HMEItem> itemlist = new ArrayList<HMEItem>();
		String ruri = null;
		URL base = null;
		String icon = null;
		String foldericon = null;
		if(request != null) {
			ruri = request.getURI();

			// get context info... based on sdk host code (Listener)
			//TODO do something with "webz" esp. vs. webz1, webz2
			IFactory factory = applistener.getFactory( "/" + WebBrowserApp.URI + "/" );
			int port = 7288;
	//logger.log(ILogger.LOG_DEBUG, "factories: "+applistener.getFactories().keySet());
			//here, uri is the hme app's uri
			//			URL uri = factory.getAssetURI(); // no, we need the BaseURI, with the app's port.
			URL uri = null;
			if(uri == null) {
				//				int port = request.getPort();
				try {
					uri = new URL("http://" + request.getInterface().getHostAddress() + ":" + port);
					String path = request.getURI();
					uri = new URL(uri, path);
				} catch (MalformedURLException e) {
					// FIXME Auto-generated catch block
					e.printStackTrace();
				}
			}
			try {
				base = new URL(uri, factory.getAppName());
			} catch (Exception e) {
				WebBrowserApp.log(request.get(V49IHmeProtocol.HTTP_HEADER_TIVO_ID), "TiVoConnect", ruri, new String[] {request.get(V49IHmeProtocol.HTTP_HEADER_TIVO_VERSION), request.get("User-Agent"), e.toString()});
			}
			// create an application context
			HostContext context = new HostContext();
			context.factory = factory;
			context.request = request;
			context.uri = uri;
			context.base = base;
			try {
				context.params = new Query(uri).getMap();
				context.appClassName = (String)factory.getFactoryData().get(IFactory.HME_APPLICATION_CLASSNAME);
			} catch (Exception e) {
				WebBrowserApp.log(request.get(V49IHmeProtocol.HTTP_HEADER_TIVO_ID), "TiVoConnect", ruri, new String[] {request.get(V49IHmeProtocol.HTTP_HEADER_TIVO_VERSION), request.get("User-Agent"), e.toString()});
			}

			// this context isn't actually an HME context which would include a
			// "tsn" header... instead we have that as TiVo_TCD_ID.
			// We need that for getpersistentdata to work correctly.
			try {
				context.params.put("tsn",
						context.request.get(V49IHmeProtocol.HTTP_HEADER_TIVO_ID));
			} catch (Exception e) {
			}

			// can use icon outline hack for HD menu in Premieres (v20 or above).
			// Premiere SD menu sends Format=text/xml in query.  HD menu doesn't but sends multiple requests
			boolean isSDMenu = false;
			if(uri.getQuery() != null)
				isSDMenu = uri.getQuery().contains("&Format=");
			String tivoVersion = request.get(V49IHmeProtocol.HTTP_HEADER_TIVO_VERSION);
			int tivoMajorVersion = 11;
			try {
				int dotIdx = tivoVersion.indexOf('.');
				if (dotIdx > 0) {
					tivoMajorVersion = Integer.parseInt(tivoVersion.substring(0, dotIdx));
				}
			} catch (Exception e) {
			}
			try {
				if (tivoMajorVersion >= 20 && !isSDMenu) {
					// Premiere icons can take up the entire entry space, so we
					// can use carefully constructed icon to outline the text or
					// have a second icon on the right or whatever.
					icon = new URL(context.base, "premiere-icon.png?" + new Date().getTime()).toString() + "=nocache";
					foldericon = new URL(context.base, "premiere-icon-folder.png?" + new Date().getTime()).toString() + "=nocache";
				} else {
					icon = new URL(context.base, "icon.png?" + new Date().getTime()).toString() + "=nocache";
					foldericon = new URL(context.base, "icon-folder.png?" + new Date().getTime()).toString() + "=nocache";
				}
			} catch (MalformedURLException e) {
			}
			
			// Include entries from user's Apps and Apps/genre folders in the bookmarks
			String bookmarkData = Bookmarks.getBookmarkData(context);
			for(String genre : genres) {
				LinkedHashMap<String, URL[]> appsg = Bookmarks.getBookmarks(Bookmarks.readablePathToFolder("Apps/"+genre+"/"), bookmarkData, context);
				if(appsg == null || appsg.size() == 0) {
					continue;
				} else {
					if(genre.equals(HMEItem.GENRE_OTHER)) {
						addItems(itemlist, context, appsg, genre, icon, foldericon, 1000+itemlist.size());
					} else {
						// use custom icons for each of the standard genres
						String genreicon = icon;
						String genrefoldericon = foldericon;
						try {
							if (tivoMajorVersion >= 20 && !isSDMenu) {
								// Premiere icons can take up the entire entry space, so we
								// can use carefully constructed icon to outline the text or
								// have a second icon on the right or whatever.
								genreicon = new URL(context.base, "premiere-icon-"+genre+".png?" + new Date().getTime()).toString() + "=nocache";
								genrefoldericon = new URL(context.base, "premiere-icon-"+genre+"-folder.png?" + new Date().getTime()).toString() + "=nocache";
							} else {
								genreicon = new URL(context.base, "icon-"+genre+".png?" + new Date().getTime()).toString() + "=nocache";
								genrefoldericon = new URL(context.base, "icon-"+genre+"-folder.png?" + new Date().getTime()).toString() + "=nocache";
							}
						} catch (MalformedURLException e) {
							// just use default icons
						}
						
						addItems(itemlist, context, appsg, genre, genreicon, genrefoldericon, 1000+itemlist.size());
					}
				}
			}

			try {
				LinkedHashMap<String, URL[]> apps = Bookmarks.getBookmarks(
						Bookmarks.readablePathToFolder("Apps/"), bookmarkData, context);
				addItems(itemlist, context, apps, null, icon, foldericon, 1000+itemlist.size());
			} catch (Exception e) {
				WebBrowserApp.log(request.get(V49IHmeProtocol.HTTP_HEADER_TIVO_ID), "TiVoConnect", ruri, new String[] {request.get(V49IHmeProtocol.HTTP_HEADER_TIVO_VERSION), request.get("User-Agent"), e.toString()});
			}
			
			Vector<Object[]> appads = Advertisement.getAppAds();
			LinkedHashMap<String, URL[]> ads = new LinkedHashMap<String, URL[]>();
			for(Object[] ad : appads) {
				try {
					String adtitle = (String) ad[0];
					URL adurl = (URL) ad[1];
					URL adicon = null;
					try {
						adicon = new URL(adurl,(String) ad[2]);
					} catch (MalformedURLException e) {
					}
					ads.put(adtitle, new URL[]{adurl, adicon});
					String genre = null;
					if(ad.length > 4 && ad[4] != null && String.valueOf(ad[4]).trim().length() > 0) {
						genre = String.valueOf(ad[4]).trim();
					}

					boolean directUrl = false;
					if(ad.length > 6 && ad[6] != null) {
						// hidden ads only used with directUrl content
						directUrl = (Boolean)ad[6];
					}
					// add them one at a time to use the Priority value
					if(ad[3] == null) //shouldn't be possible
						addItems(itemlist, context, ads, genre, icon, foldericon, 0, directUrl);
					else
						addItems(itemlist, context, ads, genre, icon, foldericon, (Integer) ad[3], directUrl);
					if(ad.length > 6 && ad[6] != null) {
						itemlist.get(itemlist.size()-1).hidden = (Boolean)ad[6];
					}
				} catch (Exception e) {
					WebBrowserApp.log(request.get(V49IHmeProtocol.HTTP_HEADER_TIVO_ID), "TiVoConnect", ruri, new String[] {request.get(V49IHmeProtocol.HTTP_HEADER_TIVO_VERSION), request.get("User-Agent"), e.toString()});
				}
				ads.clear();
			}
//			addItems(itemlist, context, ads, null, icon, foldericon, 1000+itemlist.size());
			
			// always include a base Enter Webz entry.
			boolean hasEnterWebz = false;
			String enterWebzURL = context.base.toString();
			// well, always if nothing else is left.
			hasEnterWebz = itemlist.size() > 0;
//			for(HMEItem item : itemlist) {
//				if(enterWebzURL.equals(item.url)) {
//					hasEnterWebz = true;
//					break;
//				}
//			}
////			try {
////				itemlist.add(new HMEItem("Test BG", new URL(context.base, "/tivo").toString(), null, "test-test", null));
////			} catch (MalformedURLException e) {
////				// TODO Auto-generated catch block
////				e.printStackTrace();
////			}
			if (!hasEnterWebz) {
				HMEItem enterWebzApp = new HMEItem("Enter Webz!", enterWebzURL, icon, "Enter-Webz", null);
//				enterWebzApp.topleveldisplayorderhint=0; // first
				enterWebzApp.topleveldisplayorderhint=10000; // last
				// no number = in the last group
				// if I want it first I can add it to the ads or whatever.
				itemlist.add(enterWebzApp);
			}


		}
		HMEItem[] items = new HMEItem[itemlist.size()];
		items = itemlist.toArray(items);
	//logger.log(ILogger.LOG_DEBUG, "items:"+itemlist);
			
		if (items.length == 0) {
			// last ditch effort... WBA failed (if there were no bookmarks it would have provided a default set, still)
			items = getDefaultItems(base);
			if(items == null) items = new HMEItem[0];
		}

		for(HMEItem item : items) {
			// add default icon
			if (item.iconUrl == null || item.iconUrl.trim().length() == 0) {
				item.iconUrl = icon;
			}
			//FIXME eh? strange way to do this.  pass boolean in during toString instead
			if(ruri != null && ruri.contains("DoGenres=1")) {
				item.genres=true;
			}
		}

		if(request == null) {
			WebBrowserApp.log(null, "TiVoConnect", ruri, new String[] {null, null, "items:"+items.length});
		} else {
			WebBrowserApp.log(request.get(V49IHmeProtocol.HTTP_HEADER_TIVO_ID), "TiVoConnect", ruri, new String[] {request.get(V49IHmeProtocol.HTTP_HEADER_TIVO_VERSION), request.get("User-Agent"), "items:"+items.length});
		}

		// didn't work as hoped
//		// set uuid of the Video Player
//		items[0].uuid = "863CB78F-EFDD-4106-B572-51733983DC76"; 

//logger.log(ILogger.LOG_DEBUG, "build hmeitems took millis: "+(new Date().getTime()-start1));

		return items;
	}
	
	/**
	 * Expire in at most 5 minutes so changes to bookmarks show up quickly.
	 */
	@Override
	protected int getExpirationLifespan() {
		return 1000 * 60 * 5;
	}

	private void addItems(ArrayList<HMEItem> itemlist, HostContext context, LinkedHashMap<String, URL[]> apps, String genre, String icon, String foldericon, int priority) {
		addItems(itemlist, context, apps, genre, icon, foldericon, priority, false);
	}
	private void addItems(ArrayList<HMEItem> itemlist, HostContext context, LinkedHashMap<String, URL[]> apps, String genre, String icon, String foldericon, int priority, boolean directLink) {
		if(apps == null || itemlist == null) return;
		for (String title : apps.keySet()) {
			URL[] urls = apps.get(title);
//if(itemlist.size() > 6) break;
			title = Bookmarks.folderToReadablePath(title);
			// title ending in / should be a folder
			// folders for genres are handled specially, skip them here
			if (title.endsWith("/")) {
				if (genres.contains(title.substring(0, title.length() - 1)))
					continue;
			}
			
			try {
				if (urls.length > 0) {
					String url;
					if(directLink && urls[0] != null) {
						url = urls[0].toString();
					} else {
						if(urls[0] == null) {
							url = context.base.toString(); // just launch the web browser normally
						} else {
							url = new URL( context.base, 
									"?url="+urlEncode(urls[0].toString()) ).toString();
						}
					}
					
					String itemicon = (urls.length > 1 && urls[1] != null ? urls[1].toString() : null);
					if(itemicon == null 
							|| itemicon.equals("http://feedicons.com/images/feed-icon-28x28.png") // don't use standard RSS feed icon
							|| itemicon.equals("/rss.png") // FIXME whatever the built-in is
							) {
						if(title.endsWith("/")) {
							itemicon = foldericon;
						} else {
							itemicon = icon;
						}
					}
					String uuid = 
//							title;
					 title + urls[0]; // problem with file: folder "url" newlines?
					// clean up the uuid
//					uuid = uuid.replaceAll("\\s", "-");
					uuid = uuid.replaceAll("[^0-9A-Za-z-]", "-");
//					uuid = uuid.replaceAll("[^0-9A-Fa-f-]", "-"); // real uuid is hex-based
					HMEItem item = new HMEItem(title, 
							url, 
							itemicon, 
							uuid,
							genre);
					item.topleveldisplayorderhint = priority++;
					itemlist.add(item);
				} else {
					continue;
				}
			} catch (MalformedURLException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}

	private HMEItem[] getDefaultItems(URL base) {
			if(base == null) {
				try {
					base = new URL("http://localhost:7288/webz/");
				} catch (MalformedURLException e) {
					return null;
				}
			}
			HMEItem[] items;
			items = new HMEItem[] {
					
					new HMEItem("Enter Webz!",
							base+"",
							base+"icon.png", "dlb-Enter_Webz",
							null)
					,
					new HMEItem("Youtube Website",
							base+"?url="+urlEncode("http://youtube.com"),
							"http://s.ytimg.com/yts/img/favicon_32-vflWoMFGx.png", "dlb-enterwebz-youtube.com",
							HMEItem.GENRE_VIDEO)
					,
					new HMEItem("Feature Films at Archive.org",
							base+"?url="+urlEncode("http://archive.org/details/feature_films"),
							"http://www.archive.org/images/glogo.jpg", "dlb-enterwebz-archive.org-feature_films",
							HMEItem.GENRE_VIDEO)
					,
					new HMEItem("Classic TV at Archive.org",
							base+"?url="+urlEncode("http://archive.org/details/classic_tv"),
							"http://www.archive.org/images/glogo.jpg", "dlb-enterwebz-archive.org-classic_tv",
							HMEItem.GENRE_VIDEO)
					,
					new HMEItem("Animation and Cartoons at Archive.org",
							base+"?url="+urlEncode("http://archive.org/details/animationandcartoons"),
							"http://www.archive.org/images/glogo.jpg", "dlb-enterwebz-archive.org-animationandcartoons",
							HMEItem.GENRE_VIDEO)
					,
					new HMEItem("Live Concerts at Archive.org",
							base+"?url="+urlEncode("http://archive.org/details/etree"),
							"http://www.archive.org/images/glogo.jpg", "dlb-enterwebz-archive.org-etree",
							HMEItem.GENRE_MUSIC)
					,
					new HMEItem("Radio Programs at Archive.org",
							base+"?url="+urlEncode("http://archive.org/details/radioprograms"),
							"http://www.archive.org/images/glogo.jpg", "dlb-enterwebz-archive.org-radioprograms",
							HMEItem.GENRE_MUSIC)
					,
					new HMEItem("Audio Books and Poetry at Archive.org",
							base+"?url="+urlEncode("http://archive.org/details/audio_bookspoetry"),
							"http://www.archive.org/images/glogo.jpg", "dlb-enterwebz-archive.org-audio_bookspoetry",
							HMEItem.GENRE_MUSIC)
					,
					
					new HMEItem("Weather",
							base+"?url="+urlEncode("http://m.wund.com/cgi-bin/findweather/getForecast?brand=mobile"),
							"http://icons.wunderground.com/graphics/smalllogo.gif", "dlb-enterwebz-m.wund.com",
							null)
					,
					new HMEItem("IMDB, the Internet Movie Data Base",
							base+"?url="+urlEncode("http://m.imdb.com/"),
	//						"http://i.media-imdb.com/images/SF0ae8798307d147d8ea59044184b4339e/mobile/icon_114x114.png",
							null,
							"dlb-enterwebz-imdb.com", null)
					,
					new HMEItem("TiVo's Dominos Pizza app",
							base+"?url="+urlEncode("hme://204.176.49.116:7288/dominos"),
							"http://204.176.49.116:7288/dominos.2009.05.07.r1021.logerror.signed-20090507_174812/1241743696/1289849802-1605209802/icon.png", "dlb-enterwebz-hme:dominos",
							null)
					,
					new HMEItem("Google",
							base+"?fontScale=1.5&url="+urlEncode("http://www.google.com/"),
	//						"http://www.google.com/images/google_favicon_128.png", 
							null,
							"dlb-enterwebz-www.google.com",
							null)
					,
					new HMEItem("Google Image Search",
							base+"?fontScale=1.5&url="+urlEncode("http://www.google.com/imghp"),
							null,
							"dlb-enterwebz-www.google.com-imghp",
							HMEItem.GENRE_PHOTOS)
					,
					new HMEItem("Photos.com",
							base+"?fontScale=1.5&url="+urlEncode("http://photos.com"),
							null,
							"dlb-enterwebz-photos.com",
							HMEItem.GENRE_PHOTOS)
					,
					new HMEItem("National Geographic Photography",
							base+"?fontScale=1.5&url="+urlEncode("http://photography.nationalgeographic.com"),
							null,
							"dlb-enterwebz-photo-nat-geo",
							HMEItem.GENRE_PHOTOS)
					,
					new HMEItem("Awkward Family Photos",
							base+"?fontScale=1.5&url="+urlEncode("http://awkwardfamilyphotos.com"),
							null,
							"dlb-enterwebz-awkwardfamilyphotos.com",
							HMEItem.GENRE_PHOTOS)
	
					,
					
					new HMEItem("Reversi by wmcbrine",//$NON-NLS-1$
							base+"?url="+urlEncode("hme://wmcbrine.com/reversi"), 
							"http://wmcbrine.com/images/reversi.png", //$NON-NLS-1$
							"wmcbrine-reversi-via-enterwebz",
							null)
			};
			return items;
		}

	@Override
	protected boolean handleOtherRequest(HttpRequest request) throws IOException {
		if (request.getURI().contains("EnterWebzTest")) {
	
			File testFile = new File("C:\\Users\\David\\EnterWebzTest.html");
			byte[] bytes = new byte[(int) testFile.length()];
			FileInputStream fis = new FileInputStream(testFile);
			fis.read(bytes);
			fis.close();
			request.reply(HTTP_STATUS_OK, "");
			request.addHeader("Content-Type", "text/html");
			HttpCookie testcookie = new HttpCookie("test2", "test2value");
			testcookie.setMaxAge(1000 * 100);
			testcookie.setPath("/");
			request.addHeader("Set-Cookie", testcookie.toString());
			request.addHeader("Set-Cookie", "test3=test3value");
			OutputStream outputStream = request.getOutputStream(bytes.length);
			outputStream.write(bytes);
			return true;
		} else if (request.getURI().contains("EnterWebzImg")) {
			logger.log(ILogger.LOG_DEBUG, request.getURI());
			logger.log(ILogger.LOG_DEBUG, String.valueOf(request.getInetAddress()));
			logger.log(ILogger.LOG_DEBUG, String.valueOf(request.getHeaders()));
	
			File testFile = new File("C:\\Users\\David\\EnterWebzImg.png");
			byte[] bytes = new byte[(int) testFile.length()];
			FileInputStream fis = new FileInputStream(testFile);
			fis.read(bytes);
			fis.close();
			request.reply(HTTP_STATUS_OK, "");
			request.addHeader("Content-Type", "image/png");
			OutputStream outputStream = request.getOutputStream(bytes.length);
			outputStream.write(bytes);
			return true;
		}
	
		return false;
	}
}
