package com.blackledge.david.tivo;

import java.util.ArrayList;
import java.util.Map;

import com.tivo.hme.bananas.BApplication;
import com.tivo.hme.bananas.BScreen;
import com.tivo.hme.bananas.BView;
import com.tivo.hme.sdk.Resolution;

/**
 * Simple screen to do {@link AppPassThroughView}.  Don't forget {@link #setQuitKey(int)}.  Pops when the application ends.
 * @author David
 *
 */
public class AppPassThroughScreen extends BScreen {
	
	private String appURI;
	private Map appInfo;
	private AppPassThroughView appView;
	private int quitKey = -1;
	
	public AppPassThroughScreen(BApplication app, String appURI, Map appInfo) {
		super(app);
		this.appURI = appURI;
		this.appInfo = appInfo;
	}
	
	public AppPassThroughScreen(BApplication app, String appURI, Map appInfo, Resolution theResolution) {
		super(app, theResolution);
		this.appURI = appURI;
		this.appInfo = appInfo;
	}
	
	public void setQuitKey(int key) {
		this.quitKey = key;
		if(this.appView != null) {
			if (this.quitKey > -1) {
				ArrayList<Integer> keys = new ArrayList<Integer>();
				keys.add(this.quitKey);
				this.appView.setKeyPressOverrides(keys, this);
			} else {
				this.appView.setKeyPressOverrides(null, null);
			}
		}
	}
	
	@Override
	public boolean handleKeyPress(int code, long rawcode) {
		if(code == this.quitKey) {
			getBApp().pop();
			return true;
		}
		return super.handleKeyPress(code, rawcode);
	}

	@Override
	public boolean handleEnter(Object arg, boolean isReturn) {
		if(!isReturn) {
			if(this.appView == null) {
				this.appView = new AppPassThroughView(getNormal(), 0, 0, getWidth(), getHeight());
				if (quitKey > -1) {
					ArrayList<Integer> keys = new ArrayList<Integer>();
					keys.add(this.quitKey);
					this.appView.setKeyPressOverrides(keys, this);
				}
			}
			this.setFocusDefault(appView);
			appView.setAppActions("app_started", "app_ended");
			if(!appView.isStarted()) {
				appView.startApp(appURI, appInfo);
			}
			super.handleEnter(arg, isReturn);
			return true;
		}
		super.handleEnter(arg, isReturn);
		return true;
	}
	
	@Override
	public boolean handleAction(BView view, Object action) {
		if(action.equals("app_ended")) {
			getBApp().pop();
			return true;
		} else if(action.equals("app_started")) {
			// not using this.
			return true;
		}
		return super.handleAction(view, action);
	}

}
