package com.blackledge.david.tivo;

import java.awt.Color;
import java.util.ArrayList;
import java.util.Collections;

import com.tivo.core.ds.TeDict;
import com.tivo.hme.bananas.BApplication;
import com.tivo.hme.bananas.BScreen;
import com.tivo.hme.bananas.BView;
import com.tivo.hme.interfaces.ILogger;
import com.tivo.hme.sdk.HmeEvent;
import com.tivo.hme.sdk.Resource;
import com.tivo.hme.sdk.View;

public class RemotePicker extends V49BApplication {
	
	/** set factory to report us as a V49 application */
	public static class RemotePickerFactory extends V49BApplicationFactory {}
	/** the title presented to the TiVo */
	public static final String TITLE = "Remote Picker"; //$NON-NLS-1$
	/** the URI path used by the TiVo to access this application */
	public static final String URI = "remote"; //$NON-NLS-1$

	/**
	 * Field name for TEDICT. Value should be the Remote Control Model's ID
	 * Name. Same field is returned in transitino back with user selection if
	 * something was selected.
	 */
	public static final String TEDICT_REMOTE = "REMOTE";
	
	public static class PickRemote extends BScreen {

		public String _font;
		public Color _color;
		public BView _modelView;
		public BView _remoteView;
		public BView _controlView;

		public PickRemote(BApplication bapp) {
			super(bapp);

			_font = "default-20.font";
			_color = Color.yellow;
		}
		
		private TiVoRemoteInfo getRemote() {
			return ((RemotePicker)getBApp())._currentRemote;
		}
		
		private TiVoModelInfo getModel() {
			return ((RemotePicker)getBApp())._currentModel;
		}

		@Override
		public boolean handleEnter(Object arg, boolean isReturn) {
			if (!isReturn) {
				V49BApplication app = (V49BApplication) getBApp();
				_modelView = new BView(getNormal(), app.getTitleRegion().x, app.getTitleRegion().y, 
						app.getTitleRegion().width, app.getTitleRegion().height);
				layoutModelInfo();
				_remoteView = new BView(getNormal(), app.getScrollRegion().x, app.getScrollRegion().y, 
						app.getScrollRegion().width, app.getScrollRegion().height);
				layoutRemoteInfo();
				_controlView = new BView(getNormal(), app.getSafeActionHorizontal(), app.getSafeActionVertical(), 
						getNormal().getWidth() - app.getSafeActionHorizontal()*2, getNormal().getHeight() - app.getSafeActionVertical()*2);
				layoutControls();
			}
			return super.handleEnter(arg, isReturn);
		}
		
		private void layoutRemoteInfo() {
			RemotePicker app = (RemotePicker) getApp();
			
			BView remoteImage = new BView(_remoteView, _remoteView.getWidth()/2, 0, _remoteView.getWidth()/2, _remoteView.getHeight());
			Object resource = getBApp().getResource(getRemote().getImage());
			if(resource == null) {
				resource = Color.gray;
			}
			remoteImage.setResource(resource, RSRC_IMAGE_BESTFIT|RSRC_HALIGN_CENTER|RSRC_VALIGN_CENTER);			
			
			BView remoteButtons = new BView(_remoteView, 0, 0, _remoteView.getWidth()/2, _remoteView.getHeight());
			int top[]={KEY_TIVO, /*KEY_OPT_HOME,*/ 
					/*KEY_OPT_STANDBY,*/ /*TV_POWER,*/ KEY_LIVETV, /*KEY_INPUT,*/ 
					KEY_OPT_MENU, KEY_OPT_TOP_MENU, KEY_OPT_EXIT, 
					KEY_INFO, KEY_OPT_LIST, KEY_OPT_GUIDE, KEY_OPT_DVD, 
					KEY_OPT_ANGLE, KEY_OPT_ASPECT, KEY_OPT_BACK};
			int upper[] = {KEY_UP, KEY_LEFT, KEY_RIGHT, KEY_DOWN, KEY_SELECT, 
					KEY_VOLUMEUP, KEY_VOLUMEDOWN, KEY_MUTE, 
					KEY_CHANNELUP, KEY_CHANNELDOWN};
			int middle[] = {KEY_RECORD, KEY_THUMBSDOWN, KEY_THUMBSUP, 
					KEY_PLAY, KEY_REVERSE, KEY_PAUSE, KEY_FORWARD, 
					KEY_OPT_STOP, KEY_SLOW};
			int lower[] = {KEY_REPLAY, KEY_ADVANCE, KEY_OPT_ON_DEMAND,
					KEY_OPT_ACTION_A, KEY_OPT_ACTION_B, KEY_OPT_ACTION_C, KEY_OPT_ACTION_D};
			int nums[] = {KEY_NUM1, KEY_NUM2, KEY_NUM3, 
					KEY_NUM4, KEY_NUM5, KEY_NUM6, 
					KEY_NUM7, KEY_NUM8, KEY_NUM9, 
					KEY_CLEAR, KEY_NUM0, KEY_ENTER};
			int height = remoteButtons.getHeight()/5;
			BView remoteHintsTop = new BView(remoteButtons, 0, 0, remoteButtons.getWidth(), height);
			for(int key : top) {
				app.addButton(remoteHintsTop, key, true);
			}
			BView remoteHintsUpper = new BView(remoteButtons,  0, height,remoteButtons.getWidth(), height);
			for(int key : upper) {
				app.addButton(remoteHintsUpper, key, true);
			}
			BView remoteHintsMiddle = new BView(remoteButtons, 0, height*2, remoteButtons.getWidth(), height);
			for(int key : middle) {
				app.addButton(remoteHintsMiddle, key, true);
			}
			BView remoteHintsLower = new BView(remoteButtons, 0, height*3, remoteButtons.getWidth(), height);
			for(int key : lower) {
				app.addButton(remoteHintsLower, key, true);
			}
			BView remoteHintsNums = new BView(remoteButtons, 0, height*4, remoteButtons.getWidth(), height);
			for(int key : nums) {
				app.addButton(remoteHintsNums, key, true);
			}
			
			if (app.getRemoteName() != null) {
				BView remoteTitle = new BView(_remoteView, 0, 0,
						_remoteView.getWidth(), 50);
				remoteTitle.setResource(remoteTitle.createText(_font, _color,
						app.getRemoteName()));
			}
			
		}

		private void layoutControls() {
			if (!((RemotePicker) getBApp()).atLeftExtreme()) {
				BView left = new BView(_controlView, 0, 0,
						_controlView.getWidth() / 2, _controlView.getHeight());
				left.setResource(left.createText(_font, _color, "<"),
						RSRC_HALIGN_LEFT | RSRC_VALIGN_CENTER);
			}

			if (!((RemotePicker) getBApp()).atRightExtreme()) {
				BView right = new BView(_controlView, _controlView.getWidth() / 2, 0,
						_controlView.getWidth() / 2, _controlView.getHeight());
				right.setResource(right.createText(_font, _color, ">"),
						RSRC_HALIGN_RIGHT | RSRC_VALIGN_CENTER);
			}
		}


		private void layoutModelInfo() {
			BView title = new BView(_modelView, 0, 0, _modelView.getWidth(), _modelView.getHeight());
			title.setResource(title.createText(_font, _color, "Select your current remote control:"));
			
		}

		@Override
		public boolean handleKeyPress(int code, long rawcode) {
			switch(code) {
			case KEY_LEFT:
			case KEY_REVERSE:
			case KEY_UP:
			case KEY_CHANNELUP:
				((RemotePicker)getBApp()).prevPage();
				return true;
			case KEY_RIGHT:
			case KEY_FORWARD:
			case KEY_DOWN:
			case KEY_CHANNELDOWN:
				((RemotePicker)getBApp()).nextPage();
				return true;

			case KEY_SELECT:
			case KEY_ENTER:
				((RemotePicker)getBApp()).selectPage();
				return true;

			default:
				// anything else is "cancel"
				((RemotePicker)getBApp()).cancelPage();
				return true;
			}
			
//			return super.handleKeyPress(code, rawcode);
		}

	}

	private String _remoteArg;
	private TiVoModelInfo _model;
	private TiVoRemoteInfo _remote;
	private TiVoModelInfo _currentModel;
	private TiVoRemoteInfo _currentRemote;
	private ArrayList<TiVoRemoteInfo> _remotes;
	private int _remoteIndex;
	private int _startIndex;

	@Override
	protected void showFirstPage() {
		_model = TiVoModelInfo.getInfo(getContext().getConnectionAttribute(V49IHmeProtocol.HTTP_HEADER_TIVO_ID));
		_currentModel = _model;
		if(_currentModel == null) {
			//TODO default?
		}
		_remote = null;
		if(_remoteArg != null) {
			_remote = TiVoRemoteInfo.getRemote(_remoteArg);
		}
		if(_remote == null && _currentModel != null) {
			_remote = _currentModel.getDefaultRemote();
		}
		if(_remote == null) {
			_remote = TiVoRemoteInfo.getDefaultRemote();
		}
		_currentRemote = _remote;
		
		_remotes = Collections.list(Collections.enumeration(TiVoRemoteInfo.getRemotes()));
		
		_remoteIndex = -1;
		if (_currentRemote != null) {
			_remoteIndex = _remotes.indexOf(_currentRemote);
		}
		
		if(_remoteIndex == -1) {
			_remoteIndex = 0;
		}
		_currentRemote = _remotes.get(_remoteIndex);
		
		_startIndex = _remoteIndex;
		
		push(new PickRemote(this), TRANSITION_FORWARD);
	}
	
	public String getRemoteName() {
		return _currentRemote.getName();
	}

	public void cancelPage() {
		transitionBack(null);
		setActive(false);
	}

	public void selectPage() {
		TeDict result = getPickedRemoteTeDict();
		transitionBack(result);
		setActive(false);
	}

	private TeDict getPickedRemoteTeDict() {
		String remoteName = _remotes.get(_remoteIndex).getID();
		TeDict result = new TeDict();
		result.add(RemotePicker.TEDICT_REMOTE, remoteName);
		return result;
	}

	public boolean atLeftExtreme() {
		return _remoteIndex == _startIndex;
	}
	public boolean atRightExtreme() {
		return _remoteIndex+1 == _startIndex;
	}
	
	public void prevPage() {
		int prevIndex = _remoteIndex - 1;
		if(prevIndex < 0) {
			prevIndex = _remotes.size() - 1;
		}
		
		if(atLeftExtreme()) {
			play(V49BApplication.SOUND_BONK);
			return;
		}
		
		_remoteIndex = prevIndex;
		_currentRemote = _remotes.get(_remoteIndex);
		
//		if(prevIndex < _startIndex) {
//			push(new PickRemote(this), TRANSITION_FORWARD);
//		} else {
			pop();
//		}
	}

	public void nextPage() {
		int nextIndex = _remoteIndex + 1;
		if(nextIndex > _remotes.size() - 1) {
			nextIndex = 0;
		}
		if(atRightExtreme()) {
			play(V49BApplication.SOUND_BONK);
			return;
		}
		
		_remoteIndex = nextIndex;
		_currentRemote = _remotes.get(_remoteIndex);
		
//		if(prevIndex < _startIndex) {
			push(new PickRemote(this), TRANSITION_FORWARD);
//		} else {
//			pop();
//		}
	}

	public boolean handleEvent(HmeEvent event) {
		if (event.getOpCode() == EVT_INIT_INFO) {
			HmeEvent.InitInfo info = (HmeEvent.InitInfo) event;
			debug("PARAMS:" + info.getParams()); //$NON-NLS-1$
			if(info.getParams() != null) {
				_remoteArg = info.getParams().getStringOrDefault(RemotePicker.TEDICT_REMOTE, 0, null);
			}
		}
		return super.handleEvent(event);
	}	
	
	@Override
	public void debug(String message) {
		getContext().getLogger().log(ILogger.LOG_DEBUG, message);
	}

	private static Resource getKeyResource(String keyIcon, BView resourceParent) {
		if(keyIcon == null) return null;
		String resourceName = "com/blackledge/david/tivo/keys/"+keyIcon + ".png"; //$NON-NLS-1$ //$NON-NLS-2$
		if(RemotePicker.class.getClassLoader().getResource(resourceName) == null)
			return null;
		return resourceParent.getResource(resourceName);
	}
	
	private static Resource getDefaultKeyResource(BView resourceParent) {
		String defaultResource = "com/blackledge/david/tivo/keys/KEY-background.png"; //$NON-NLS-1$
		return resourceParent.getResource(defaultResource);
	}
	
	private BView addButton(BView hintParent, int key, boolean horizontal) {
		return addButton(_currentRemote, hintParent, key, horizontal);
	}

	/** Adds Button to end of parent view, laying it out to the side or below the last child view.  returns last child created */
	public static BView addButton(TiVoRemoteInfo remote, BView hintParent, int key, boolean horizontal) {
		return addHint(remote, hintParent, new int[]{key}, null, null, horizontal);
		//TODO make addHint do a special case for a single key and no text to reduce views
	}

//	TiVoModelInfo model = TiVoModelInfo.getInfo(hintParent.getBApp().getContext().getConnectionAttribute("tsn")); //$NON-NLS-1$
//	TiVoRemoteInfo remote = null;
//	if(model != null) {
//		remote = model.getDefaultRemote();
//	}
//	if(remote == null) {
//		remote = TiVoRemoteInfo.getDefaultRemote();
//	}
	
	public static BView addHint(TiVoRemoteInfo remote, BView hintParent, int[] keys, String text, Color textColor, boolean horizontal) {
		return addHint(remote, hintParent, keys, text, null, -1, textColor, -1, horizontal);
	}
	/** Adds Button Hint to end of parent view, laying it out to the side or below the last child view.  returns last child created */
	public static BView addHint(TiVoRemoteInfo remote, BView hintParent, int[] keys, String text, String textFont, int textHeight, Color textColor, int width, boolean horizontal) {
		int x = 0; int y = 0;
		BView parent = hintParent;//TODO maybe use a hint group parent
		int hintHeight = 11;
		if (textHeight > 0) {
			hintHeight = textHeight+2;
		}
		int hintWidth = 50;
		if (width > 0) {
			hintWidth = width;
		}
		int hintImageHeight = 16;
		int hintImageWidth = 50;
		if(horizontal) {
			// keep 0, 0
		} else {
			x = parent.getWidth() - hintWidth;
		}
		
		View lastChild = null;
		View lastButton = null;
		if (hintParent.getChildCount() > 0) {
			lastChild = hintParent.getChild(hintParent.getChildCount() - 1);
			if (hintParent.getChildCount() > 1 && text != null) {
				lastButton = hintParent.getChild(hintParent.getChildCount() - 2);
			} else {
				lastButton = lastChild;//TODO this really isn't true, just a patch for now
			}

			if(horizontal) {
				x = lastChild.getX()+lastChild.getWidth();
				y = lastButton.getY();
				// start a new row if needed.
				if (x + hintWidth > hintParent.getWidth()) {
					y += lastChild.getHeight() + lastButton.getHeight();
					x = 0;
					lastChild = null;
				}
			} else {
				x = lastChild.getX();
				y = lastChild.getY()+lastChild.getHeight();
				// start a new column if needed.
				if (y + hintHeight + hintImageHeight > hintParent.getHeight()) {
					x -= lastChild.getWidth();
					y = 0;
					lastChild = null;
				}
			}
		}
		
		String hintfont = "default-12.font"; //$NON-NLS-1$
		if(textFont != null) {
			hintfont = textFont;
		}
		Color hintColor = Color.white;
		if (textColor != null) {
			hintColor = textColor;
		}
		
		// track whether we actually made a button hint
		boolean button = false;
		
		int keysStarted = 0;
		int keyTop = 0;

		for(int key : keys) {
			if(remote == null || ! remote.isKeyPresent(key))
				continue;
			
			Resource image;
			String keytext = remote.getKeyName(key);
			String keyicon = remote.getKeyIcon(key);
			if(keyicon != null) {
				image = getKeyResource(keyicon, hintParent);
				if(image == null) {
					//FIXME need to do more than just this...need more text if we're stuck with default image.
					// default
					image = getDefaultKeyResource(hintParent);
				}
			} else {
				// no icon or text, doesn't exist.
				if(keytext == null) {
					image = null;
				} else {
					// default
					image = getDefaultKeyResource(hintParent);
				}
			}
			
			if(keytext != null || image != null) {
				boolean topText = keytext != null && keytext.endsWith("\\n");
				boolean botText = keytext != null && keytext.startsWith("\\n");
//					int newlines = 0;
//					// count the "newlines", but don't count them if they're both top and bottom
//					if (keytext != null && !(topText && botText)) {
//						int index = keytext.indexOf("\\n");
//						while (index > -1) {
//							++newlines;
//							index = keytext.indexOf("\\n", index + 1);
//						}
//					}

				int h = (image != null ? 
//							hintImageHeight + (newlines * hintHeight)
				hintImageHeight + (topText || botText ? hintHeight : 0)
						: hintHeight);
				int nextKeyTop = keyTop;
				++keysStarted;
				if(keysStarted == 1) {
					lastChild = new BView(parent, 
							x
							+(hintWidth-hintImageWidth),
							y,
							hintImageWidth, h);

					lastChild.setResource(Color.black);
				} else {
					lastChild.setSize(lastChild.getWidth(), lastChild.getHeight()+h);
				}
				
				button = true;
				
				if(image != null) {
					BView imageChild = new BView((BView) lastChild,
							0, keyTop 
//								,
							+ (topText?
//										lastChild.getHeight()-hintImageHeight
									hintHeight
									:0),
//								(keysStarted-1)*hintImageHeight+(topText?lastChild.getHeight()-hintImageHeight:0),
							lastChild.getWidth(), hintImageHeight);
					if(topText) {
						imageChild.setResource(image, 
								RSRC_IMAGE_VFIT| RSRC_HALIGN_CENTER | RSRC_VALIGN_BOTTOM);
					} else if(botText) {
						imageChild.setResource(image, 
								RSRC_IMAGE_VFIT| RSRC_HALIGN_CENTER | RSRC_VALIGN_TOP);
					} else {
						imageChild.setResource(image, 
								RSRC_IMAGE_VFIT| RSRC_HALIGN_CENTER | RSRC_VALIGN_CENTER);
					}
					nextKeyTop = Math.max(nextKeyTop, imageChild.getY()+imageChild.getHeight());
				}
				if(keytext != null) {
					keytext = keytext.replace("\\n\\n", "\n");
					keytext = keytext.replace("\\n", "\n");
//						if(keytext.endsWith("\\n"))keytext = keytext.substring(0, keytext.length() - 2);
//						if(keytext.startsWith("\\n"))keytext = keytext.substring(2);
					keytext = keytext.trim();
					BView textChild = new BView((BView) lastChild,
							0, keyTop,
//								(keysStarted-1)*hintImageHeight,
							lastChild.getWidth(), h);
//								lastChild.getHeight()-(keysStarted-1)*hintImageHeight);
					if(topText) {
						textChild.setResource(textChild.createText("system-8.font", Color.white, keytext),  //$NON-NLS-1$
								RSRC_HALIGN_CENTER | RSRC_VALIGN_TOP);
					} else if(botText) {
						textChild.setResource(textChild.createText("system-8.font", Color.white, keytext),  //$NON-NLS-1$
								RSRC_HALIGN_CENTER | RSRC_VALIGN_BOTTOM);
					} else {
					textChild.setResource(textChild.createText("system-8.font", Color.white, keytext),  //$NON-NLS-1$
							RSRC_HALIGN_CENTER | RSRC_VALIGN_CENTER);
					}
					
					nextKeyTop = Math.max(nextKeyTop, textChild.getY()+textChild.getHeight());
				}
				keyTop = nextKeyTop;

				x = lastChild.getX();
				x -= (hintWidth-hintImageWidth);
				y = lastChild.getY()+lastChild.getHeight();
				
			}
		}
		
		// if we didn't get a button hint, don't show the text, either.
		if(text != null && button) {
			//count the number of text lines
			int lines = 1;
			int lastline = -1;
			lastline = text.indexOf('\n', lastline + 1);
			while(lastline > -1) {
				++lines;
				lastline = text.indexOf('\n', lastline + 1);
			}
			
			lastChild = new BView(parent, 
					x,y,
//						lastChild.getX(), lastChild.getY()+lastChild.getHeight(),
					hintWidth, hintHeight*lines);
			lastChild.setResource(lastChild.createText(hintfont, hintColor, text), RSRC_HALIGN_RIGHT|RSRC_VALIGN_TOP);
			
		}

		if (lastChild instanceof BView)
			return (BView) lastChild;
		else
			return null;
	}

	public static TiVoRemoteInfo getRemoteForTiVo(String tivoID) {
		//FIXME get last user-selected remote
		TiVoRemoteInfo remote = TiVoRemoteInfo.getRemote(tivoID);
		if(remote == null) {
			return TiVoRemoteInfo.getDefaultRemote();
		} else {
			return remote;
		}
	}
	
	
}
