package com.blackledge.david.tivo.blitz;

import java.awt.Color;
import java.awt.Rectangle;
import java.net.MalformedURLException;
import java.net.URL;
//import java.text.SimpleDateFormat;
import java.util.Calendar;

import com.blackledge.david.tivo.RemotePicker;
import com.blackledge.david.tivo.TiVoRemoteInfo;
import com.blackledge.david.tivo.V49BApplication;
import com.blackledge.david.tivo.V49BApplicationFactory;
import com.blackledge.david.tivo.V49IHmeProtocol;
import com.tivo.hme.bananas.BScreen;
import com.tivo.hme.bananas.BView;

/**
simple Airnow browser - directions to browse various wide shots (ch up/dn to change map type):
925x925
 * 
 * @author David
 *
 */
public class Lightning extends V49BApplication {

	private static class LightningScreen extends BScreen {

		private BView _leftHint;
		private BView _rightHint;
		private BView _upHint;
		private BView _downHint;
		private BView _dateText;
//		private BView _reverseHint;
//		private BView _forwardHint;
		private BView _image;

		public LightningScreen(Lightning airQualityApp) {
			super(airQualityApp);
		}
		
		@Override
		public boolean handleEnter(Object arg, boolean isReturn) {
			if (!isReturn) {

				Lightning app = ((Lightning)getBApp());
				
				loadLocation(app);
				
				TiVoRemoteInfo remote = RemotePicker.getRemoteForTiVo(app.tivoID);
				if(remote == null) remote = TiVoRemoteInfo.getDefaultRemote();

				Rectangle region = app.getScrollRegion();
				int hintHeight = 30;
				int hintWidth = 30;
				_leftHint = new BView(getAbove(),
//						region.x-hintWidth, region.y+(region.height - hintHeight) / 2, 
						region.x, region.y+(region.height - hintHeight) / 2, 
						hintWidth, hintHeight);
				RemotePicker.addHint(remote, _leftHint,
						new int[] { KEY_LEFT }, null, null, hintHeight, null, hintWidth, true);
				_rightHint = new BView(getAbove(),
//						region.x + region.width, region.y+(region.height - hintHeight) / 2,
						region.x + region.width-hintWidth, region.y+(region.height - hintHeight) / 2,
						hintWidth, hintHeight);
				RemotePicker.addHint(remote, _rightHint,
						new int[] { KEY_RIGHT }, null, null, hintHeight, null, hintWidth, true);
				_upHint = new BView(getAbove(),
//						region.x + (region.width - hintWidth)/2, region.y-hintHeight,
						region.x + (region.width - hintWidth)/2, region.y,
						hintWidth, hintHeight);
				RemotePicker.addHint(remote, _upHint,
						new int[] { KEY_UP }, null, null, hintHeight, null, hintWidth, true);
				_downHint = new BView(getAbove(),
//						region.x + (region.width - hintWidth)/2, region.y+region.height,
						region.x + (region.width - hintWidth)/2, region.y+region.height-hintHeight,
						hintWidth, hintHeight);
				RemotePicker.addHint(remote, _downHint,
						new int[] { KEY_DOWN }, null, null, hintHeight, null, hintWidth, true);

				//FIXME add blitzortung logo?
//				Rectangle logoRegion = app.getLogoRegion();
//				BView logo = new BView(getAbove(), logoRegion.x, logoRegion.y, logoRegion.width, logoRegion.height);
//				logo.setResource(logo.getResource("http://airnow.gov/images/airnow-logo-150x81.jpg"), RSRC_IMAGE_BESTFIT|RSRC_VALIGN_TOP|RSRC_HALIGN_CENTER);
//				
//				logo = new BView(getAbove(), logoRegion.x, logoRegion.y, logoRegion.width, logoRegion.height);
//				logo.setResource(logo.getResource("http://airnow.gov/images/aqi_ex1.jpg"), RSRC_IMAGE_BESTFIT|RSRC_VALIGN_BOTTOM|RSRC_HALIGN_CENTER);
				
				getBelow().setResource(getResource(STREAM_LOOP_GREEN_NOWPLAYING));
				
				Rectangle titleRegion = app.getTitleRegion();
//				_reverseHint = new BView(getAbove(), 
//						titleRegion.x, 
//						titleRegion.y + (titleRegion.height - hintHeight) / 2,
//						hintWidth, hintHeight);
//				RemotePicker.addHint(remote, _reverseHint,
//						new int[] { KEY_REVERSE }, null, null, hintHeight, null, hintWidth, true);
				_dateText = new BView(getNormal(), 
						titleRegion.x + hintWidth, 
						titleRegion.y,
						titleRegion.width - hintWidth*2, titleRegion.height);
//				_forwardHint = new BView(getAbove(), 
//						titleRegion.x + titleRegion.width - hintWidth, 
//						titleRegion.y + (titleRegion.height - hintHeight) / 2,
//						hintWidth, hintHeight);
//				RemotePicker.addHint(remote, _forwardHint,
//						new int[] { KEY_FORWARD }, null, null, hintHeight, null, hintWidth, true);
				
				_image = new BView(getNormal(),
						region.x, region.y,
						region.width, region.height);
			}
			
			setHints();
//			setDateHints();		
			_dateText.setResource(_dateText.createText("default-40.font", Color.white, "Blitzortung.org Lightning"));
			setImage();
			return true;
		}

		private void storeLocation() {
			Lightning app = (Lightning) getBApp();
			String regionStore = app._region_row+"|"+app._region_col;
			app.getContext().setPersistentData(REGION_DATA, regionStore);
		}
		
		private void loadLocation(Lightning app) {
			String regionStore = app.getContext().getPersistentData(REGION_DATA);
			if(regionStore != null && regionStore.contains("|")) {
				int i = regionStore.indexOf("|");
				app._region_row = Integer.parseInt( regionStore.substring(0,i));
				app._region_col = Integer.parseInt( regionStore.substring(i+1));
			}
		}

		private void setHints() {
			Lightning app = ((Lightning)getBApp());
			
			_rightHint.setVisible(app._region_col != app.getNextHorizontal(1));
			_leftHint.setVisible(app._region_col != app.getNextHorizontal(-1));
			_upHint.setVisible(app._region_row != app.getNextVertical(-1));
			_downHint.setVisible(app._region_row != app.getNextVertical(1));
		}
		
		private void setImage() {
			Lightning app = ((Lightning)getBApp());
			
//			if(_image.getResource() != null)
//				_image.getResource().remove();
			_image.setResource(_image.getResource(app.getImageURL()),
					RSRC_HALIGN_CENTER | RSRC_VALIGN_CENTER 
					//FIXME bestfit includes legend, but shrinks under most resolutions.
//					| RSRC_IMAGE_BESTFIT
					| RSRC_IMAGE_HFIT
					);
		}

//		private void setDateHints() {
//			Lightning app = ((Lightning)getBApp());
//			
//			if (_dateText.getResource() != null)
//				_dateText.getResource().remove();
//			_dateText.setResource(_dateText.createText("default-40.font", Color.white, app.getDateText()));
//			
//			_forwardHint.setVisible(!app.isForecast());
//			
//		}
		
		@Override
		public boolean handleKeyRepeat(int code, long rawcode) {
			switch(code) {
			case KEY_UP:
			case KEY_DOWN:
			case KEY_LEFT:
			case KEY_RIGHT:
//			case KEY_REVERSE:
//			case KEY_FORWARD:
				return handleKeyPress(code, rawcode);
			}
			return super.handleKeyRepeat(code, rawcode);
		}
		
		@Override
		public boolean handleKeyPress(int code, long rawcode) {
			Lightning app = (Lightning) getBApp();
			
			switch(code) {
			case KEY_UP:
				int up = app.getNextVertical(-1);
				if (up != app._region_row) {
					app.play(SOUND_UPDOWN);
					app._region_row = up;
					storeLocation();
					setHints();
					setImage();
				}
				return true;
			case KEY_DOWN:
				int down = app.getNextVertical(1);
				if (down != app._region_row) {
					app.play(SOUND_UPDOWN);
					app._region_row = down;
					storeLocation();
					setHints();
					setImage();
				}
				return true;
			case KEY_RIGHT:
				int right = app.getNextHorizontal(1);
				if (right != app._region_col) {
					app.play(SOUND_UPDOWN);
					app._region_col = right;
					storeLocation();
					setHints();
					setImage();
				}
				return true;
			case KEY_LEFT:
				int left = app.getNextHorizontal(-1);
				if (left != app._region_col) {
					app.play(SOUND_UPDOWN);
					app._region_col = left;
					storeLocation();
					setHints();
					setImage();
				}
				return true;
				
//			case KEY_FORWARD:
//				if(!app.isForecast()) {
//					app.play(SOUND_SPEEDUP1);
//					app._selected_date.add(Calendar.DAY_OF_MONTH, 1);
//					setDateHints();		
//					setImage();
//					return true;
//				}
//				break;
//			case KEY_REVERSE:
//				app.play(SOUND_SLOWDOWN1);
//				app._selected_date.add(Calendar.DAY_OF_MONTH, -1);
//				setDateHints();		
//				setImage();
//				return true;
				
			case V49IHmeProtocol.KEY_OPT_ZOOM:
			case KEY_REPLAY:
			case KEY_PAUSE:
			case KEY_CLEAR:
				app.play(SOUND_DESELECT);
				app.setActive(false);
				return true;
			}
			return super.handleKeyPress(code, rawcode);
		}

		
	}

	String[][] _regions = {
			{"us", "eu", "as", "us"},
			{"sa", null, "oc", "sa"}
	};
	
	public static final String REGION_DATA = "REGION";
	
	public static class LightningFactory extends V49BApplicationFactory {
	}
	
//	// year, month, day, location
//	private static final String FORECAST_URL_FORMAT = "http://www.epa.gov/airnow/today/forecast_aqi_%1$04d%2$02d%3$02d_%4$s.jpg";
	// year, month, day, location
	private static final String TODAY_URL_FORMAT = "http://images.blitzortung.org/Images/image_b_%4$s.png";
//	// year, month, day, location
//	private static final String PAST_URL_FORMAT = "http://www.epa.gov/airnow/%1$04d/%1$04d%2$02d%3$02d/peak_aqi_%4$s.jpg";
	
	int _region_row = 0;
	int _region_col = 0;
	Calendar _date = Calendar.getInstance();
	Calendar _selected_date = (Calendar) _date.clone();
	
	@Override
	protected void showFirstPage() {

		push(new LightningScreen(this), TRANSITION_FORWARD);
		
	}

	@Override
	public void playSoundForKey(int code, boolean wasHandled, boolean inScreen) {
//		switch(code) {
//		case KEY_LEFT:
//		case KEY_RIGHT:
//		case KEY_UP:
//		case KEY_DOWN:
//			break;
//		}
		// TODO Auto-generated method stub
		super.playSoundForKey(code, wasHandled, inScreen);
	}
	
//	public boolean isForecast() {
//		return _date.before(_selected_date);
//	}

//	SimpleDateFormat _dateFormat = new SimpleDateFormat("MMMM dd, yyyy");
	
//	public String getDateText() {
//		if (_date.equals(_selected_date)) {
//			return "Current Conditions";
//		} else if (_date.before(_selected_date)) {
//			return "Today's Forecast";
//		} else {
//			return _dateFormat.format(_selected_date.getTime());
//		}
//	}

	private int getNextHorizontal(int dir) {
		String cur_region = _regions[_region_row][_region_col];
		int col = _region_col;
		int row = _region_row;
		col += dir;
		while(validLocation(row, col)) {
			if(_regions[row][col] != null && !_regions[row][col].equals(cur_region)) {
				return col;
			}
			col += dir;
		}
		return _region_col;
	}

	private boolean validLocation(int row, int col) {
		return !(row >= _regions.length || col >= _regions[0].length
				|| row < 0 || col < 0);
	}
	
	private int getNextVertical(int dir) {
		String cur_region = _regions[_region_row][_region_col];
		int col = _region_col;
		int row = _region_row;
		row += dir;
		while(validLocation(row, col)) {
			if(_regions[row][col] != null && !_regions[row][col].equals(cur_region)) {
				return row;
			}
			row += dir;
		}
		return _region_row;
	}
	
	private URL getImageURL() {
		if (_region_row >= _regions.length || _region_col > _regions[0].length
				|| _region_row < 0 || _region_col < 0) {
			_region_row = 0;
			_region_col = 0;
		}
		String region = _regions[_region_row][_region_col];
		if(region == null) {
			return null;
		}
		String url_s;
//		if (_date.equals(_selected_date)) {
			// use TODAY for the url fields
			int year = _date.get(Calendar.YEAR);
			int month = _date.get(Calendar.MONTH) + 1;
			int day = _date.get(Calendar.DAY_OF_MONTH);
			url_s = String.format(TODAY_URL_FORMAT, year, month, day, region);
//		} else if (_date.before(_selected_date)) {
//			// use TODAY for the url fields
//			int year = _date.get(Calendar.YEAR);
//			int month = _date.get(Calendar.MONTH) + 1;
//			int day = _date.get(Calendar.DAY_OF_MONTH);
//			url_s = String.format(FORECAST_URL_FORMAT, year, month, day, region);
//		} else {
//			int year = _selected_date.get(Calendar.YEAR);
//			int month = _selected_date.get(Calendar.MONTH) + 1;
//			int day = _selected_date.get(Calendar.DAY_OF_MONTH);
//			url_s = String.format(PAST_URL_FORMAT, year, month, day, region);
//		}
		
		try {
			URL url = new URL(url_s);
			return url;
		} catch (MalformedURLException e) {
			e.printStackTrace();
			return null;
		}
	}

}
