package com.blackledge.david.tivo.game;

import java.awt.Color;
import java.awt.Rectangle;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Vector;

import com.blackledge.david.game.MultiplayerGame;
import com.blackledge.david.game.PlayerState;
import com.blackledge.david.game.MultiplayerGame.GameData;
import com.blackledge.david.game.MultiplayerGame.GameInfo;

import com.blackledge.david.tivo.game.Game.GameFactory;

import com.tivo.hme.bananas.BApplication;
import com.tivo.hme.bananas.BList;
import com.tivo.hme.bananas.BScreen;
import com.tivo.hme.bananas.BView;
import com.tivo.hme.sdk.Resolution;

public class GameLobby extends BScreen {

		private GameLobby.GameList list;
		private GameInfo[] _lastGameList;
		private BView players;
		public GameLobby(BApplication app) {
			super(app);
			init();
		}

		public GameLobby(BApplication app, Resolution theResolution) {
			super(app, theResolution);
			init();
		}

		public static class GameList extends BList {

		    public GameList(BView parent, int x, int y, int width, int height, int rowHeight) {
		    	super(parent, x, y, width, height, rowHeight);
		    	setBarAndArrows(BAR_DEFAULT, BAR_HANG, "quit", "join");
			}
		    
		    @Override
		    public boolean handleKeyPress(int code, long rawcode) {
		    	switch(code) {
		    	case KEY_SELECT:
		    		join(false);
		    		return true;
		    	}
		    	return super.handleKeyPress(code, rawcode);
		    }
		    
		    @Override
		    public boolean handleAction(BView view, Object action) {
		    	if(action.equals("join")) {
		    		return join(false);
		    	} else if(action.equals("quit")) {
		    		return quit();
		    	}
		    	return super.handleAction(view, action);
		    }

			private boolean quit() {
				getBApp().setActive(false);
				return true;
			}

			private boolean join(boolean spectate) {
				if(getFocus() < 0 || getFocus() >= size()) {
					return false;
				}
				GameInfo gameInfo = (GameInfo) get(getFocus());
				if(!spectate) {
					GameData data = ((GameFactory) getBApp().getFactory()).getGame().getGameData(gameInfo.gameID);
					if(data != null && gameInfo.players.size() >= data.maximumPlayers) {
						spectate = true;
					}
				}
				try {
					((Game) getBApp()).setSpectating(spectate);

					((Game) getBApp()).joinGame(gameInfo.gameID, spectate);
				} catch (Exception e) {
					return false;
				}
				return true;
			}

			@Override
			protected void createRow(BView parent, int index) {
				GameFactory factory = (GameFactory) getBApp().getFactory();
				if(get(index) instanceof GameInfo) {
					GameInfo gameInfo = (GameInfo) get(index);
					
					int playerCount = gameInfo.players.size();
					String players = playerListString(gameInfo, playerCount);

					String labelString;
					GameData game = factory.getGame().getGameData(gameInfo.gameID);
					if(gameInfo.gameID == MultiplayerGame.GAME_VS_COMPUTER) {
						labelString = "Start a Game against the Computer";
					} else if(playerCount <= 0 || gameInfo.gameID == MultiplayerGame.GAME_NEW_GAME) {
						labelString = "Start New Game and Wait for Players";
					} else if(game != null && playerCount < game.minimumPlayers) {
						if (playerCount == 1)
							labelString = "Start Game with player " + players;
						else
							labelString = "Start Game with " + players;
					} else if(game == null || playerCount < game.maximumPlayers) {
						if (playerCount == 1)
							labelString = "Join Game with player "+players;
						else
							labelString = "Join Game with "+players;
					} else {
						if (playerCount == 1)
							labelString = "Watch Game of "+players;
						else
							labelString = "Watch Game between "+players;
					}
					
					parent.setResource(parent.createText("default-"+(getRowHeight()-2)+".font", Color.white, 
							"   "+labelString), RSRC_HALIGN_LEFT|RSRC_VALIGN_CENTER);
				}
			}

			private String playerListString(GameInfo gameInfo, int playerCount) {
				String players = "";
				if(playerCount > 0) {
					players += gameInfo.players.get(0).name;
					if(playerCount > 1) {
						if(playerCount == 2) {
							players += " and ";
						} else {
							players += ", ";
						}
						players += gameInfo.players.get(1).name;
						if(playerCount > 2) {
							players += ", ("+(playerCount-2)+" more)";
						}
					}
				}
				return players;
			}
		}
		
		protected void init() {
			this.getBelow().setResource(getResource("com/blackledge/david/tivo/game/table1.gif"), RSRC_HALIGN_LEFT|RSRC_VALIGN_CENTER|RSRC_IMAGE_VFIT);
			Rectangle scrollRegion = ((Game)getBApp()).getScrollRegion();
			players = new BView(getNormal(), scrollRegion.x, scrollRegion.y, scrollRegion.width, 40);
			list = new GameList(getNormal(), scrollRegion.x, scrollRegion.y+players.getHeight(), scrollRegion.width, scrollRegion.height-players.getHeight(), 
					(int) (20*((Game)getBApp()).getHeightMultiplier()));
			setFocusDefault(list);
		}

		void update() {
			GameFactory factory = (GameFactory) getBApp().getFactory();
			
			List<PlayerState> idle = factory.getIdlePlayers();
			idle.remove(((Game)getBApp())._player);
			String playerText;
			if(idle.size() == 0) {
				playerText = "Idle players will be listed in this message.";
			} else if(idle.size() == 1) {
				playerText = idle.get(0)+" is idle and not playing.";
			} else {
				playerText = idle.size() + " people are idle and not playing: "+ idle;
			}
			if(players.getResource() != null) {
				players.getResource().remove();
				players.clearResource();
			}
			players.setResource(players.createText("default-17-italic.font", Color.yellow, playerText));
			
			GameInfo[] gameList = factory.getGame().getGameList();
			
			if(_lastGameList != null && Arrays.equals( _lastGameList, gameList)) {
				return;
			}
			_lastGameList = gameList;
			
			int oldFocus = list.getFocus();
			if(oldFocus < 0) oldFocus = 0;
			list.clear();
//			this.bookmarkStartIndex = 3;//list.size();

			GameInfo newgame = new GameInfo();
			newgame.gameID = MultiplayerGame.GAME_NEW_GAME;
			newgame.players = new ArrayList<PlayerState>();
			list.add(newgame);
			
			if (factory.getGame().isComputerPlayerAvailable()) {
				GameInfo computergame = new GameInfo();
				computergame.gameID = MultiplayerGame.GAME_VS_COMPUTER;
				computergame.players = new ArrayList<PlayerState>();
				list.add(computergame);
			}
			
			if (gameList != null) {
				list.add(new Vector<GameInfo>(Arrays.asList(gameList)));
			}
			
			if(oldFocus >= list.size()) {
				oldFocus = list.size()-1;
			}
			list.setFocus(oldFocus, false);
		}
		
		@Override
		public boolean handleKeyPress(int code, long rawcode) {
			switch(code) {
			case KEY_CLEAR:
				getBApp().setActive(false);
				return true;
			}
			return super.handleKeyPress(code, rawcode);
		}
		
		@Override
		public boolean handleEnter(Object arg, boolean isReturn) {
			((Game)getBApp()).startLobbyUpdates();
			((Game)getBApp()).setSpectating(false);
			
			return super.handleEnter(arg, isReturn);
		}
		
		@Override
		public boolean handleExit() {
			((Game)getBApp()).stopLobbyUpdates();
			
			return super.handleExit();
		}
	}