package com.blackledge.david.tivo.game;

import java.awt.Color;
import java.awt.Point;
import java.awt.Rectangle;
import java.util.ArrayList;
import java.util.Arrays;

import com.blackledge.david.game.BoardState;
import com.blackledge.david.game.GameException;
import com.blackledge.david.game.MultiplayerGame;
import com.blackledge.david.game.MultiplayerGame.GameData;
import com.blackledge.david.game.PlayerState;

import com.tivo.hme.bananas.BEvent;
import com.tivo.hme.bananas.BView;

public class SampleGame extends Game {

	public static class SampleGameScreen extends GameScreen {

		private SampleGame app;

		public SampleGameScreen(SampleGame sampleGame) {
			super(sampleGame);
			app = sampleGame;
			
			SampleGameFactory game = (SampleGameFactory) getBApp().getFactory();
			
			Rectangle region = ((Game)getBApp()).getScrollRegion();
			int offx = region.x;
			int offy = region.y;
			int wid = region.width / game.getBoardWidth();
			int hei = region.height / game.getBoardHeight();
			int xpad = (int) (wid * .1);
			int ypad = (int) (hei * .1);
			wid = (int) (wid * .9);
			hei = (int) (hei * .9);
			
			_boardFocus = new BView(getBelow(), 0, 0, wid+xpad*2, hei+ypad*2);
			
			_boardView = new BView[game.getBoardWidth()][game.getBoardHeight()];
			for (int x = 0; x < game.getBoardWidth(); ++x) {
				for (int y = 0; y < game.getBoardHeight(); ++y) {
					_boardView[x][y] = new BView(getNormal(), 
							offx + x * (wid + xpad), 
							offy + y * (hei + ypad), 
							wid, 
							hei);
					_boardView[x][y].setFocusable(true);
				}
			}
			
			
			setFocusDefault(_boardView[0][0]);
			
		}

		private BView[][] _boardView;
		private BView _boardFocus;
		
		@Override
		protected void updateGameBoard(BoardState gameBoard) {
			if(_boardView == null) return;
			for(int x = 0 ; x < gameBoard.width ; ++x) {
				for(int y = 0 ; y < gameBoard.height ; ++y) {
					Color color = null;
					int val = gameBoard.get(x,y);
					if(val != -1) {
						color = new Color(val);
					}
					if (!(color == null && _boardView[x][y].getResource() == null)) {
						_boardView[x][y].setResource(color);
					}
				}
			}
		}

		@Override
		protected void updateGamePlayers(ArrayList<PlayerState> gamePlayers) {
			super.updateGamePlayers(gamePlayers);
		}
		
		@Override
		public void setFocus(BView focus) {
			super.setFocus(focus);
			if(focus != null && _boardFocus != null) {
				_boardFocus.setLocation(
						focus.getX()
						- ((_boardFocus.getWidth() - focus.getWidth()) / 2),
						focus.getY()
						- ((_boardFocus.getHeight() - focus.getHeight()) / 2),
						getResource("*50"));
				
				// translate focus location to game position, and set player position.
				for(int x = 0 ; x < _boardView.length ; ++x) {
					for (int y = 0 ; y < _boardView[x].length ; ++y) {
						if(_boardView[x][y] == focus) {
							notifyPlayerPosition(x, y);
						}
					}
				}
				
			}
		}

		private void notifyPlayerPosition(int x, int y) {
			PlayerState player = getPlayer();
			player.position = new Point(x,y);
			((GameFactory)(getBApp().getFactory())).playerUpdate(getGameID(), player);
		}

		private int getGameID() {
			return app.getGameID();
		}

		private PlayerState getPlayer() {
			return ((Game)getBApp())._player;
		}
		
		@Override
		public boolean handleEnter(Object arg, boolean isReturn) {
			boolean result = super.handleEnter(arg, isReturn);
			_boardFocus.setResource(getPlayer().color);
			_boardFocus.setTransparency(.20f);
			return result;
		}
		
		@Override
		public boolean handleKeyPress(int code, long rawcode) {
			SampleGameFactory game = (SampleGameFactory) app.getFactory();
			switch (code) {
			case KEY_UP:
				postEvent(new BEvent.Action(getFocus(), H_UP));
				return true;
			case KEY_DOWN:
				postEvent(new BEvent.Action(getFocus(), H_DOWN));
				return true;
			case KEY_LEFT:
				postEvent(new BEvent.Action(getFocus(), H_LEFT));
				return true;
			case KEY_RIGHT:
				postEvent(new BEvent.Action(getFocus(), H_RIGHT));
				return true;
			case KEY_SELECT:
				for(int x = 0; x < _boardView.length ; ++x) {
					for(int y = 0 ; y < _boardView[x].length ; ++y) {
						if(_boardView[x][y] == getFocus()) {
							Point point = new Point(x,y);
							if(game.playerAction(getGameID(), getPlayer(),
									SampleGameFactory.ACTION.SELECT.ordinal(),
									point)) {
								play(SOUND_SELECT);
								app.updateGameNow();
							} else {
								play(SOUND_BONK);
							}
							return true;
						}
					}
				}
				return true;
			case KEY_THUMBSDOWN:
				game.playerChat(getGameID(), getPlayer(), "No.");
				return true;
			case KEY_THUMBSUP:
				game.playerChat(getGameID(), getPlayer(), "Yes.");
				return true;
			case KEY_PLAY:
				game.playerChat(getGameID(), getPlayer(), "Play, already.");
				return true;
			case KEY_PAUSE:
				game.playerChat(getGameID(), getPlayer(), "Hold On.");
				return true;
			}
			return super.handleKeyPress(code, rawcode);
		}

		@Override
		public boolean handleKeyRepeat(int code, long rawcode) {
			switch(code) {
			case KEY_UP:
			case KEY_DOWN:
			case KEY_RIGHT:
			case KEY_LEFT:
				return handleKeyPress(code, rawcode);
			}
			return super.handleKeyRepeat(code, rawcode);
		}
	}

	public static class SampleGameFactory extends GameFactory {

		public static enum ACTION {SELECT};
		
		@Override
		public boolean playerAction(int gameID, PlayerState playerIdentification,
				int action, Object details) {
			ACTION act;
			try {
				act = ACTION.values()[action];
			} catch (Exception e) {
				return false;
			}
			switch(act) {
			case SELECT:
				// must have details of a Point that was selected
				if (playerIdentification.hasTurn && details != null) {
					Point location = (Point) details;
					
					int val = getGame().getGameBoard(gameID).get(location.x, location.y);
					if (val == -1) {
						getGame().getGameBoard(gameID).set(location.x, location.y,
								playerIdentification.color.getRGB());
						nextTurn(getGame().getGameData(gameID), playerIdentification);
						getGame().boardUpdated(gameID);
						getGame().playersUpdated(gameID);
						return true;
					} else if (val == playerIdentification.color.getRGB()) {
						getGame().getGameBoard(gameID).set(location.x, location.y, -1);
						nextTurn(getGame().getGameData(gameID), playerIdentification);
						getGame().boardUpdated(gameID);
						getGame().playersUpdated(gameID);
						return true;
					} else {
						return false;
					}
				}
				return false;
			}
			return false;
		}

		@Override
		public synchronized int newGame() {
			int newGame = super.newGame();
			GameData result = getGame().getGameData(newGame);
			result.minimumPlayers = 1;
			result.maximumPlayers = colors.length;
			return newGame;
		}

		public int getBoardHeight() {
			return 10;
		}

		public int getBoardWidth() {
			return 15;
		}

		private Color[] colors = new Color[]{Color.blue.brighter(), Color.red, Color.green, Color.cyan};
		
		@Override
		protected void addPlayer(GameData game, PlayerState player)
				throws GameException {
			synchronized(colors) {
				// player may not enter with a color, it must be assigned
				player.color = null;

				// assign colors to all players
				ArrayList<Color> c = new ArrayList<Color>();
				c.addAll(Arrays.asList(colors));
				// don't use any previous players' colors
				for(PlayerState p : game.players) {
					c.remove(p.color);
				}
				System.out.println("REMAINING COLORS:"+c);
//				// assign colors to any outstanding other players
//				for(PlayerState p : game.players) {
//					if(p.color == null) {
//						p.color = c.remove(0);
//					}
//					if(player.equals(p)) {
//						player.color = p.color;
//					}
//				}
				if(player.color == null) {
					player.color = c.remove(0);
				}
			}
			
			super.addPlayer(game, player);
		}
		
		@Override
		protected void removePlayer(GameData game, PlayerState player) {
			Color removed = player.color;
			// remove player's info
			super.removePlayer(game, player);
			// remove player's plays.
			if(removed != null) {
				int gone = removed.getRGB();
				for(int x = 0 ; x < game.boardState.width ; ++x) {
					for(int y = 0 ; y < game.boardState.height ; ++y) {
						if(game.boardState.get(x,y) == gone) {
							game.boardState.set(x,y,-1);
						}
					}
				}
				getGame().boardUpdated(game.gameID);
			}
		}

		@Override
		public MultiplayerGame getGame() {
			// TODO Auto-generated method stub
			return null;
		}
	}

	private SampleGameScreen _gameScreen;

	@Override
	protected GameLobby newGameLobby(Game app) {
		return new GameLobby(this);
	}
	
	@Override
	public void joinGame(int gameID, boolean spectate) {
		//TODO support spectate
		
		_gameScreen = new SampleGameScreen(this);
		
		
		
		push(_gameScreen, TRANSITION_FORWARD, gameID);
		
	}
	
}
