package com.blackledge.david.tivo.game.chainreaction;

import java.awt.Color;
import java.awt.Point;
import java.awt.Rectangle;
import java.util.ArrayList;
import java.util.List;

import com.blackledge.david.game.BoardState;
import com.blackledge.david.game.MultiplayerGame.GameData;
import com.blackledge.david.game.PlayerState;

import com.blackledge.david.tivo.game.Game;
import com.blackledge.david.tivo.game.GameScreen;
import com.blackledge.david.tivo.game.chainreaction.PigeonHole.PigeonHoleFactory;

import com.blackledge.david.game.chainreaction.ChainReactionGame;
import com.blackledge.david.game.chainreaction.ChainReactionGame.ACTION;

import com.tivo.hme.bananas.BView;
import com.tivo.hme.sdk.Resource;

import com.blackledge.david.tivo.RemotePicker;
import com.blackledge.david.tivo.TiVoRemoteInfo;
import com.blackledge.david.tivo.V49BApplication;

public class PigeonHoleGameScreen extends GameScreen {

	private BView _helpView;
	private BView _gameArea;
//	private BView _notice;
	private int _focusIndex = 0;
	private int _focusIndey = 0;
	private BView _focusView;
	private BView[][] _spaceView;
	private boolean _animatingReaction;
	private List<BoardState> _reactions = new ArrayList<BoardState>();

	public PigeonHoleGameScreen(Game app) {
		super(app);
		
		Rectangle gameArea = getGameRegion();
		
		_gameArea = new BView(getNormal(), gameArea.x, gameArea.y, gameArea.width, gameArea.height);

		// FIXME calc sizes based on gamearea and board size
		int cellwidth = 50;
		int cellheight = 50;
		
		// focusview is first child, so it will be below the other items.
		_focusView = new BView(_gameArea, 0, 0, cellwidth+4, cellheight+4, false);
		_focusView.setResource(Color.yellow);

		PigeonHoleFactory factory = (PigeonHoleFactory)app.getFactory();
		_spaceView = new BView[factory.getBoardWidth()][factory.getBoardHeight()];
		for(int x = 0 ; x < _spaceView.length ; ++x) {
			for(int y = 0 ; y < _spaceView[0].length ; ++y) {
				_spaceView[x][y] = new BView(_gameArea, cellwidth*x, cellheight*y, cellwidth, cellheight);
			}
		}
		
		

//		_notice = new BView(getAbove(), gameArea.x, gameArea.y
//				/*+gameArea.height/2-70*/, gameArea.width, 70, false);
//		new BView(_notice, 0, 0, _notice.getWidth(), _notice.getHeight());
//		_notice.setResource(new Color(0, 0, 50));
		
		
		Rectangle titleArea = getGameTitleRegion();
		BView title = new BView(getNormal(), titleArea.x, titleArea.y, titleArea.width, titleArea.height);
		title.setResource(title.createText("default-30.font", Color.white, "Pigeon Hole"), RSRC_VALIGN_TOP|RSRC_HALIGN_CENTER);
		
		BView hint = new BView(getNormal(), titleArea.x, titleArea.y+titleArea.height/2, titleArea.width, titleArea.height/2);
		TiVoRemoteInfo remote = RemotePicker.getRemoteForTiVo(((V49BApplication)app).tivoID);
		RemotePicker.addHint(remote, hint, new int[] {KEY_CLEAR}, "Quit Match\n", Color.yellow, true);
		RemotePicker.addHint(remote, hint, new int[] {KEY_INFO}, "Help/Rules\n", Color.yellow, true);
		RemotePicker.addHint(remote, hint, new int[] {KEY_SELECT}, "Place Pigeon\n", Color.yellow, true);
		
		
		_helpView = new BView(getAbove(), gameArea.x, gameArea.y, gameArea.width, gameArea.height, false);
		_helpView.setResource(Color.DARK_GRAY);
		_helpView.setTransparency(0.10f);
		BView text = new BView(_helpView, 0, 0, _helpView.getWidth(), _helpView.getHeight());
		int helpFontSize = (int) (15 * ((Game)getBApp()).getHeightMultiplier());
		text.setResource(text.createText("default-"+helpFontSize+"-italic.font", Color.white, "" +
				"Pigeon Hole Help\n" +
				"\n" +
				"Place a pigeon in any pigeon hole.\n" +
				"\n" +
				"If the hole gets overcrowded with pigeons, " +
				"they will all join your flock and fly to neighboring holes.\n" +
				"\n" +
				"Corner holes can only hold 1 pigeon, sides can hold 2, " +
				"and others can hold 3 before overcrowding.\n" +
				"\n" +
				"You win when all of the pigeons are in your flock.\n" +//FIXME and all holes have a pigeon?
				""), RSRC_VALIGN_TOP|RSRC_HALIGN_CENTER|RSRC_TEXT_WRAP);
		
	}
	
	@Override
	public boolean handleEnter(Object arg, boolean isReturn) {
		boolean result = super.handleEnter(arg, isReturn);
//		getBelow().setResource(V49BApplication.STREAM_LOOP_BLUE_SETUP);
		getBelow().setResource(getResource(Color.blue.darker().darker()));
		return result;
	}
	
	@Override
	public boolean handleExit() {
		if (getBelow().getResource() != null) {
			getBelow().getResource().remove();
			getBelow().clearResource();
		}
		return super.handleExit();
	}

	@Override
	protected void updateGameBoard(BoardState gameBoard) {

		PigeonHoleFactory mainGame = (PigeonHoleFactory) getBApp().getFactory();
		PigeonHole app = (PigeonHole) getBApp();
		GameData gameData = getGameData();
		
		ChainReactionGame game = (ChainReactionGame)mainGame.getGame();
		List<BoardState> reaction = game.getReaction(app.getGameID());
		if(reaction != null && reaction.size() > 1) {
			animateReactions(reaction, game.getPlayerWithTurn(app.getGameID()));
		}
//		for(int x = 0 ; x < gameBoard.width ; ++x) {
//			for(int y = 0 ; y < gameBoard.height ; ++y) {
//			}
//		}
	}

	
	private void animateReactions(List<BoardState> reaction, PlayerState player) {
		if (_reactions != null) {
			synchronized (_reactions) {
				_reactions.addAll(reaction);

				if (!_animatingReaction) {
					startAnimation(player);
				}
			}
		}
	}

	public void processAnim(int arg) {
		startAnimation(getGameData().players.get(arg));
	}

	private void startAnimation(PlayerState player) {
		synchronized(_reactions) {
			while(_reactions.size() > 0) {
				_animatingReaction = true;
				BoardState reaction = _reactions.remove(0);
				// get entry... if it has explosions:
				if(isExplosive(reaction)) {
					freeAnimViews();
					Resource anim = getResource("*400,-0.50");
					//  begin animating explosions and player-centric flights
					for(int x = 0 ; x < reaction.width ; ++x) {
						for(int y = 0 ; y < reaction.height ; ++y) {
							if(reaction.get(x, y) == -100) {
								explode(player, reaction, x, y, anim);
							}
						}
					}

					//  schedule next update at end of animation if anything else.
					int arg = getGameData().players.indexOf(player);
					((PigeonHole)getBApp()).scheduleAnimation(anim, arg);
					break;
				} else {
					// if it doesn't have explosions:
					//  update views immediately
					updateBoard(reaction);
					//FIXME two of these in a row means we changed players during animation.
					//  loop
				}
			}
			if(_reactions.size() == 0) {
				_animatingReaction = false;
			}
		}
	}

	private void updateBoard(BoardState reaction) {
		//FIXME fill all slots in with player icons
		
		ChainReactionGame game = (ChainReactionGame) ((PigeonHoleFactory)((PigeonHole)getBApp()).getFactory()).getGame();
		for(int x = 0 ; x < reaction.width ; ++x) {
			for(int y = 0 ; y < reaction.height ; ++y) {
				int val = reaction.get(x, y);
				int[] players = game.getPlayerValues(val);
				String string = "/";
				for(int i = 0 ; i < players.length ; ++i) {
					string += players[i]+"/";
				}
				_spaceView[x][y].setResource(createText("default-15.font", Color.white, string));
			}
		}
		
	}

	private void explode(PlayerState player, BoardState reaction, int x,
			int y, Resource anim) {
		// animate explosion
		BView center = _spaceView[x][y];
		BView exp = getAnimView();
		exp.setLocation(center.getX(), center.getY());
		exp.setResource(getExplosionResource());
		exp.setVisible(true);
		exp.setVisible(false, anim);
		if(x > 0) {
			// animate to left
			BView fly = getAnimView();
			fly.setLocation(center.getX(), center.getY());
			fly.setResource(getPigeonFlightResource(player));
			fly.setVisible(true);
			fly.setVisible(false, anim);
			fly.setLocation(center.getX()-center.getWidth(), center.getY(), anim);
		}
		if(x < reaction.width-1) {
			// animate right
			BView fly = getAnimView();
			fly.setLocation(center.getX(), center.getY());
			fly.setResource(getPigeonFlightResource(player));
			fly.setVisible(true);
			fly.setVisible(false, anim);
			fly.setLocation(center.getX()+center.getWidth(), center.getY(), anim);
		}
		if(y > 0) {
			// animate up
			BView fly = getAnimView();
			fly.setLocation(center.getX(), center.getY());
			fly.setResource(getPigeonFlightResource(player));
			fly.setVisible(true);
			fly.setVisible(false, anim);
			fly.setLocation(center.getX(), center.getY()-center.getHeight(), anim);
		}
		if(y < reaction.height-1) {
			// animate down
			BView fly = getAnimView();
			fly.setLocation(center.getX(), center.getY());
			fly.setResource(getPigeonFlightResource(player));
			fly.setVisible(true);
			fly.setVisible(false, anim);
			fly.setLocation(center.getX(), center.getY()+center.getHeight(), anim);
		}
	}

	private Resource getExplosionResource() {
		//FIXME get animated image or something
		return getResource(Color.red);
	}
	
	private Resource getPigeonFlightResource(PlayerState player) {
		//FIXME get animated image or something
		return getResource(player.color==null?Color.gray:player.color);
	}

	private List<BView> _animViews = new ArrayList<BView>();
	private int _animViewIndex = 0;
	private BView getAnimView() {
		synchronized (_animViews) {
			if (_animViewIndex >= _animViews.size()) {
				_animViews.add(new BView(_gameArea, 0, 0, _spaceView[0][0]
						.getWidth(), _spaceView[0][0].getHeight(), false));
			}
			return _animViews.get(_animViewIndex++);
		}
	}
	private void freeAnimViews() {
		_animViewIndex = 0;
	}

	private boolean isExplosive(BoardState reaction) {
		for(int x = 0 ; x < reaction.width ; ++x) {
			for(int y = 0 ; y < reaction.height ; ++y) {
				if(reaction.get(x,y) == -100) {
					return true;
				}
			}
		}
		return false;
	}

	@Override
	public boolean handleKeyPress(int code, long rawcode) {
		PigeonHoleFactory mainGame = (PigeonHoleFactory) getBApp().getFactory();
		PigeonHole app = (PigeonHole) getBApp();
		if(app.isSpectating()) {
			switch(code) {
			case KEY_LEFT:
				leaveGame();
				return true;
			case KEY_INFO:
				_helpView.setVisible(!_helpView.getVisible());
				return true;
			}
			return super.handleKeyPress(code, rawcode);
		}
		GameData gameData = getGameData();
		// get the game instance's player instance
		PlayerState player = gameData.players.get(gameData.players.indexOf(app.getPlayer()));
//		PlayerState player = app.getPlayer();
		
		ACTION action;
		action = ChainReactionGame.ACTION.ADD;
//		switch(gameData.gameState) {
//		case DISCARDS:
//			action = ChainReactionGame.ACTION.ADD;
//			break;
//		case ROUND_BEGIN:
//			action = PigeonHoleGame.ACTION.DEAL;
//			break;
//		case GO_NEXT:
//			action = PigeonHoleGame.ACTION.GO;
//			break;
//		}
		switch(code) {
		case KEY_LEFT:
			getBApp().play(app.SOUND_UPDOWN);
			return focusOn(_focusIndex - 1, _focusIndey);
		case KEY_RIGHT:
			getBApp().play(app.SOUND_UPDOWN);
			return focusOn(_focusIndex + 1, _focusIndey);
		case KEY_UP:
			getBApp().play(app.SOUND_UPDOWN);
			return focusOn(_focusIndex, _focusIndey - 1);
		case KEY_DOWN:
			getBApp().play(app.SOUND_UPDOWN);
			return focusOn(_focusIndex, _focusIndey + 1);
		case KEY_SELECT:
		case KEY_PLAY:
			getBApp().play(app.SOUND_UPDOWN);
			return actOn(mainGame, app, player, action, _focusIndex, _focusIndey);
		case KEY_INFO:
			_helpView.setVisible(!_helpView.getVisible());
			return true;
		}
		
		return super.handleKeyPress(code, rawcode);
	}

	private boolean actOn(PigeonHoleFactory mainGame, PigeonHole app,
			PlayerState player, ACTION action, int index, int indey) {
		// adjust focus if needed (just to be safe)
		focusOn(index, indey);
		boolean result = mainGame.playerAction(app.getGameID(), player, action.ordinal(), new Point(_focusIndex, _focusIndey));
		// adjust focus if needed (not unlikely)
		focusOn(_focusIndex, _focusIndey);
		return result;
	}
	
	private boolean focusOn(int i, int j) {
		PigeonHole app = (PigeonHole) getBApp();
		PigeonHoleFactory mainGame = (PigeonHoleFactory) app.getFactory();

		if (_focusIndex != i) {
			_focusIndex = i;
			updateFocus();
			return true;
		}
		if (_focusIndey != j) {
			_focusIndey = j;
			updateFocus();
			return true;
		}
		return false;
	}

	private void updateFocus() {
		if(_focusIndex >= 0 && _focusIndex < _spaceView.length && _focusIndey >= 0 && _focusIndey < _spaceView[0].length) {
			Resource anim = getResource("*250,0.25");
			_focusView.setVisible(true, anim);
			_focusView.setLocation(_spaceView[_focusIndex][_focusIndey].getX()-2, _spaceView[_focusIndex][_focusIndey].getY()-2, anim);
		}
	}

	@Override
	protected void updateGamePlayers(ArrayList<PlayerState> gamePlayers) {
		
		super.updateGamePlayers(gamePlayers);
		
	}


}
